# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack.common import tag


class LoadBalancer(resource.Resource, tag.TagMixin):
    resource_key = 'loadbalancer'
    resources_key = 'loadbalancers'
    base_path = '/lbaas/loadbalancers'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'description', 'flavor_id', 'name', 'project_id', 'provider',
        'vip_address', 'vip_network_id', 'vip_port_id', 'vip_subnet_id',
        'vip_qos_policy_id', 'provisioning_status', 'operating_status',
        'availability_zone', is_admin_state_up='admin_state_up',
        **tag.TagMixin._tag_query_parameters
    )

    # Properties
    #: The administrative state of the load balancer *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Name of the target Octavia availability zone
    availability_zone = resource.Body('availability_zone')
    #: Timestamp when the load balancer was created
    created_at = resource.Body('created_at')
    #: The load balancer description
    description = resource.Body('description')
    #: The load balancer flavor ID
    flavor_id = resource.Body('flavor_id')
    #: List of listeners associated with this load balancer
    listeners = resource.Body('listeners', type=list, list_type=dict)
    #: The load balancer name
    name = resource.Body('name')
    #: Operating status of the load balancer
    operating_status = resource.Body('operating_status')
    #: List of pools associated with this load balancer
    pools = resource.Body('pools', type=list, list_type=dict)
    #: The ID of the project this load balancer is associated with.
    project_id = resource.Body('project_id')
    #: Provider name for the load balancer.
    provider = resource.Body('provider')
    #: The provisioning status of this load balancer
    provisioning_status = resource.Body('provisioning_status')
    #: Timestamp when the load balancer was last updated
    updated_at = resource.Body('updated_at')
    #: VIP address of load balancer
    vip_address = resource.Body('vip_address')
    #: VIP netowrk ID
    vip_network_id = resource.Body('vip_network_id')
    #: VIP port ID
    vip_port_id = resource.Body('vip_port_id')
    #: VIP subnet ID
    vip_subnet_id = resource.Body('vip_subnet_id')
    # VIP qos policy id
    vip_qos_policy_id = resource.Body('vip_qos_policy_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Flavor(resource.Resource):
    base_path = '/flavors'

    resources_key = 'flavors'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'region', 'engine_name')

    region = resource.URI('region')
    # Properties
    #: Indicates the engine name.
    engine_name = resource.Body('engine_name')
    #: Indicates the node type. DDS contains the following types of nodes:
    # * mongos
    # * shard
    # * config
    # * replica
    type = resource.Body('type')
    #: Number of vCPUs.
    vcpus = resource.Body('vcpus')
    #: Indicates the memory size in gigabyte (GB).
    ram = resource.Body('ram')
    #: Indicates the resource specifications code.
    spec_code = resource.Body('spec_code')
    #: Indicates the status of specifications in an AZ.
    az_status = resource.Body('az_status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.dis.v2 import checkpoint
from otcextensions.sdk.dis.v2 import checkpoint as sdk_checkpoint
from otcextensions.tests.unit.osclient.dis.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestCreateCheckpoint(fakes.TestDis):

    columns = (
        'app_name',
        'checkpoint_type',
        'partition_id',
        'sequence_number',
        'stream_name',
    )

    def setUp(self):
        super(TestCreateCheckpoint, self).setUp()

        self.cmd = checkpoint.CreateCheckpoint(self.app, None)

    def test_create(self):
        arglist = [
            'test-stream',
            'test-app',
            '--partition-id', '1',
            '--sequence-number', '1',
            '--checkpoint-type', 'LAST_READ',
        ]
        verifylist = [
            ('streamName', 'test-stream'),
            ('appName', 'test-app'),
            ('partition_id', '1'),
            ('sequence_number', '1'),
            ('checkpoint_type', 'LAST_READ'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        attrs = {
            'stream_name': 'test-stream',
            'app_name': 'test-app',
            'partition_id': '1',
            'sequence_number': '1',
            'checkpoint_type': 'LAST_READ'
        }

        self.client.create_checkpoint = mock.Mock(
            return_value=sdk_checkpoint.Checkpoint(**attrs))

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_checkpoint.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestShowCheckpoint(fakes.TestDis):
    _data = fakes.FakeCheckpoint.create_one()

    columns = ('metadata', 'sequence_number')
    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowCheckpoint, self).setUp()

        self.cmd = checkpoint.ShowCheckpoint(self.app, None)

        self.client.get_checkpoint = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            'test-stream',
            'test-app',
            '--partition-id', '1',
            '--checkpoint-type', 'LAST_READ',
        ]

        verifylist = [
            ('streamName', 'test-stream'),
            ('appName', 'test-app'),
            ('partition_id', '1'),
            ('checkpoint_type', 'LAST_READ'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_checkpoint.assert_called_with(
            stream_name='test-stream',
            app_name='test-app',
            partition_id='1',
            checkpoint_type='LAST_READ'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteCheckpoint(fakes.TestDis):

    _data = fakes.FakeCheckpoint.create_multiple(2)

    def setUp(self):
        super(TestDeleteCheckpoint, self).setUp()

        self.client.delete_checkpoint = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = checkpoint.DeleteCheckpoint(self.app, None)

    def test_delete(self):
        arglist = [
            'test-stream',
            'test-app',
            '--partition-id', '1',
            '--checkpoint-type', 'LAST_READ',
        ]

        verifylist = [
            ('streamName', 'test-stream'),
            ('appName', 'test-app'),
            ('partition_id', '1'),
            ('checkpoint_type', 'LAST_READ'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_checkpoint.assert_called_with(
            stream_name='test-stream',
            app_name='test-app',
            partition_id='1',
            checkpoint_type='LAST_READ'
        )

        self.assertIsNone(result)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""AS Groups v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


def set_attributes_for_print_detail(obj):
    info = {}  # instance._info.copy()
    info['name'] = obj.name
    info['id'] = obj.id
    info['create_time'] = obj.create_time
    instance_config = obj.instance_config
    if instance_config:
        info['instance_name'] = instance_config.instance_name
        info['instance_id'] = instance_config.instance_id
        info['flavor_id'] = instance_config.flavor_id
        info['image_id'] = instance_config.image_id
        info['key_name'] = instance_config.key_name
        info['public_ip'] = instance_config.public_ip
        info['user_data'] = instance_config.user_data
        info['metadata'] = instance_config.metadata
        disks = []
        for disk in instance_config.disk:
            dsk = {
                'size': disk['size'],
                'volume_type': disk['volume_type'],
                'disk_type': disk['disk_type'],
            }
            disks.append(dsk)
        info['disk'] = disks

    return info


class ListAutoScalingGroup(command.Lister):
    _description = _('List AutoScaling Groups')
    columns = ('ID', 'Name', 'status', 'detail')

    def get_parser(self, prog_name):
        parser = super(ListAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name or ID of the AS group')
        )
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run')
        )
        parser.add_argument(
            '--scaling-configuration-id',
            metavar='<scaling_configuration_id>',
            help=_('ID of the AS configuration')
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Shows AS groups with specific status:\n'
                   '<INSERVICE>: AS group is working\n'
                   '<PAUSED>: AS group is paused\n'
                   '<ERROR>: AS group has malfunctions\n'
                   '<DELETING>: AS group is being deleted')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        args = {}
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.marker:
            args['marker'] = parsed_args.marker
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.scaling_configuration_id:
            args['scaling_configuration_id'] = \
                parsed_args.scaling_configuration_id
        if parsed_args.status:
            args['status'] = parsed_args.status

        data = client.groups(**args)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowAutoScalingGroup(command.ShowOne):
    _description = _('Shows details of a AutoScalinig group')
    columns = ['ID', 'Name', 'Status', 'Detail',
               'scaling_configuration_id', 'scaling_configuration_name',
               'current_instance_number', 'desire_instance_number',
               'min_instance_number', 'max_instance_number',
               'cool_down_time', 'networks', 'available_zones',
               'security_group']

    def get_parser(self, prog_name):
        parser = super(ShowAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('ID or name of the configuration group')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        obj = client.find_group(parsed_args.group, ignore_missing=False)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})

        return (display_columns, data)


class CreateAutoScalingGroup(command.ShowOne):
    _description = _('Creates AutoScalinig group')
    columns = ['ID', 'Name', 'Status', 'Detail',
               'Datastore Version Name', 'Is Scaling']

    def get_parser(self, prog_name):
        parser = super(CreateAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the new configuration group')
        )
        parser.add_argument(
            '--desire-instance-number',
            metavar='<desire_instance_number>',
            type=int,
            help=_('Desired number of instances')
        )
        parser.add_argument(
            '--min-instance-number',
            metavar='<min_instance_number>',
            type=int,
            help=_('Minimal number of instances')
        )
        parser.add_argument(
            '--max-instance-number',
            metavar='<max_instance_number>',
            type=int,
            help=_('Maximal number of instances')
        )
        parser.add_argument(
            '--cool-down-time',
            metavar='<cool_down_time>',
            type=int,
            help=_('Specifies cooling duration in seconds')
        )
        parser.add_argument(
            '--lb-listener-id',
            metavar='<lb_listener_id>',
            help=_('Specifies ELB Listener ID')
        )
        parser.add_argument(
            '--lbaas-listener',
            metavar='<lbaas_listener>',
            action='append',
            help=_('Specifies ULB Listener Information in format: '
                   'ID:PORT:Weight(optional) '
                   '(Repeat multiple times, up to 3 times)')
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            action='append',
            help=_('Specifies the availability zones information '
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--network-id',
            metavar='<network_id>',
            action='append',
            required=True,
            help=_('Network ID of the subnet'
                   '(Repeat multiple times, up to 5 times)')
        )
        parser.add_argument(
            '--security-group',
            metavar='<security_group>',
            action='append',
            # required=True,
            help=_('Security Group ID'
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            required=True,
            help=_('Router (VPC) ID')
        )
        parser.add_argument(
            '--audit-method',
            metavar='<audit_method>',
            help=_('Specifies the audit method [`NOVA_AUDIT`, `ELB_AUDIT`]')
        )
        parser.add_argument(
            '--audit-time',
            metavar='<audit_time>',
            type=int,
            help=_('Specifies the audit time in minutes')
        )
        parser.add_argument(
            '--terminate-policy',
            metavar='<terminate_policy>',
            help=_('Specifies the termination policy'
                   ' [`OLD_CONFIG_OLD_INSTANCE` (default), '
                   '`OLD_CONFIG_NEW_INSTANCE`, '
                   '`OLD_INSTANCE`, '
                   '`NEW_INSTANCE`]')
        )
        parser.add_argument(
            '--notification',
            metavar='<notification>',
            action='append',
            help=_('Specifies the notification method (`EMAIL` for Email) '
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--delete-public-ip',
            default=False,
            action='store_true',
            help=_('Specifies whether to delete EIP when deleting the ECS')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}
        args['name'] = parsed_args.name
        args['router_id'] = parsed_args.router_id

        networks = []
        for network in parsed_args.network_id:
            networks.append({'id': network})
        args['networks'] = networks

        if parsed_args.security_group:
            sgs = []
            for sg in parsed_args.security_group:
                sgs.append({'id': sg})
            args['security_groups'] = sgs

        if parsed_args.desire_instance_number:
            args['desire_instance_number'] = parsed_args.desire_instance_number
        if parsed_args.max_instance_number:
            args['max_instance_number'] = parsed_args.max_instance_number
        if parsed_args.min_instance_number:
            args['min_instance_number'] = parsed_args.min_instance_number
        if parsed_args.cool_down_time:
            args['cool_down_time'] = parsed_args.cool_down_time
        if parsed_args.lb_listener_id:
            args['lb_listener_id'] = parsed_args.lb_listener_id
        if parsed_args.lbaas_listener:
            listeners = []
            for lsnr in parsed_args.lbaas_listener:
                lb_parts = lsnr.split(':')
                lb = {
                    'id': lb_parts[0],
                    'protocol_port': lb_parts[1]
                }
                if len(lb_parts) == 3:
                    lb['weight'] = lb_parts[2]
                listeners.append(lb)
            args['lbaas_listeners'] = listeners
        if parsed_args.availability_zone:
            zones = []
            for zone in parsed_args.availability_zone:
                zones.append(zone)
            # zones.append(zone for zone in parsed_args.availability_zone)
            args['available_zones'] = zones
        if parsed_args.audit_method:
            args['health_periodic_audit_method'] = parsed_args.audit_method
        if parsed_args.audit_time:
            args['health_periodic_audit_time'] = parsed_args.audit_time
        if parsed_args.terminate_policy:
            args['instance_terminate_policy'] = parsed_args.terminate_policy
        if parsed_args.delete_public_ip:
            args['delete_public_ip'] = parsed_args.delete_public_ip
        if parsed_args.notification:
            lst = []
            for notification in parsed_args.notification:
                lst.append(notification)
            args['notifications'] = lst

        client = self.app.client_manager.auto_scaling
        group = client.create_group(**args)
        display_columns, columns = _get_columns(group)
        data = utils.get_item_properties(group, columns, formatters={})

        return (display_columns, data)


class DeleteAutoScalingGroup(command.Command):
    _description = _('Deletes AutoScalinig group')
    columns = ['ID', 'Name', 'Status', 'Detail',
               'Datastore Version Name', 'Is Scaling']

    def get_parser(self, prog_name):
        parser = super(DeleteAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('ID or name of the configuration group to be deleted')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling
        if parsed_args.group:
            client.delete_group(parsed_args.group, ignore_missing=False)


class UpdateAutoScalingGroup(command.ShowOne):
    _description = _('Updates AutoScalinig group')

    columns = ['ID', 'Name', 'Status', 'Detail',
               'Datastore Version Name', 'Is Scaling']

    def get_parser(self, prog_name):
        parser = super(UpdateAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('AS Group name or ID')
        )
        parser.add_argument(
            '--desire-instance-number',
            metavar='<desire_instance_number>',
            type=int,
            help=_('Desired number of instances')
        )
        parser.add_argument(
            '--min-instance-number',
            metavar='<min_instance_number>',
            type=int,
            help=_('Minimal number of instances')
        )
        parser.add_argument(
            '--max-instance-number',
            metavar='<max_instance_number>',
            type=int,
            help=_('Maximal number of instances')
        )
        parser.add_argument(
            '--cool-down-time',
            metavar='<cool_down_time>',
            type=int,
            help=_('Specifies cooling duration in seconds')
        )
        parser.add_argument(
            '--lb-listener-id',
            metavar='<lb_listener_id>',
            help=_('Specifies ELB Listener ID')
        )
        parser.add_argument(
            '--lbaas-listener',
            metavar='<lbaas_listener>',
            action='append',
            help=_('Specifies ULB Listener Information in format: '
                   'ID:PORT:Weight(optional) '
                   '(Repeat multiple times, up to 3 times)')
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            action='append',
            help=_('Specifies the availability zones information '
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--network-id',
            metavar='<network_id>',
            default=[],
            action='append',
            help=_('Network ID of the subnet'
                   '(Repeat multiple times, up to 5 times)')
        )
        parser.add_argument(
            '--security-group',
            metavar='<security_group>',
            default=[],
            action='append',
            help=_('Security Group ID'
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            help=_('Router (VPC) ID')
        )
        parser.add_argument(
            '--audit-method',
            metavar='<audit_method>',
            help=_('Specifies the audit method [`NOVA_AUDIT`, `ELB_AUDIT`]')
        )
        parser.add_argument(
            '--audit-time',
            metavar='<audit_time>',
            type=int,
            help=_('Specifies the audit time in minutes')
        )
        parser.add_argument(
            '--terminate-policy',
            metavar='<terminate_policy>',
            help=_('Specifies the termination policy'
                   ' [`OLD_CONFIG_OLD_INSTANCE` (default), '
                   '`OLD_CONFIG_NEW_INSTANCE`, '
                   '`OLD_INSTANCE`, '
                   '`NEW_INSTANCE`]')
        )
        parser.add_argument(
            '--notification',
            metavar='<notification>',
            action='append',
            help=_('Specifies the notification method (`EMAIL` for Email) '
                   '(Repeat multiple times)')
        )
        parser.add_argument(
            '--delete-public-ip',
            default=False,
            action='store_true',
            help=_('Specifies whether to delete EIP when deleting the ECS')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.router_id:
            args['router_id'] = parsed_args.router_id

        networks = []
        for network in parsed_args.network_id:
            networks.append({'id': network})
        if networks:
            args['networks'] = networks

        sgs = []
        for sg in parsed_args.security_group:
            sgs.append({'id': sg})
        if sgs:
            args['security_groups'] = sgs

        if parsed_args.desire_instance_number:
            args['desire_instance_number'] = parsed_args.desire_instance_number
        if parsed_args.max_instance_number:
            args['max_instance_number'] = parsed_args.max_instance_number
        if parsed_args.min_instance_number:
            args['min_instance_number'] = parsed_args.min_instance_number
        if parsed_args.cool_down_time:
            args['cool_down_time'] = parsed_args.cool_down_time
        if parsed_args.lb_listener_id:
            args['lb_listener_id'] = parsed_args.lb_listener_id
        if parsed_args.lbaas_listener:
            listeners = []
            for lsnr in parsed_args.lbaas_listener:
                lb_parts = lsnr.split(':')
                lb = {
                    'id': lb_parts[0],
                    'protocol_port': lb_parts[1]
                }
                if len(lb_parts) == 3:
                    lb['weight'] = lb_parts[2]
                listeners.append(lb)
            args['lbaas_listeners'] = listeners
        if parsed_args.availability_zone:
            zones = []
            for zone in parsed_args.availability_zone:
                zones.append(zone)
            # zones.append(zone for zone in parsed_args.availability_zone)
            args['available_zones'] = zones
        if parsed_args.audit_method:
            args['health_periodic_audit_method'] = parsed_args.audit_method
        if parsed_args.audit_time:
            args['health_periodic_audit_time'] = parsed_args.audit_time
        if parsed_args.terminate_policy:
            args['instance_terminate_policy'] = parsed_args.terminate_policy
        if parsed_args.delete_public_ip:
            args['delete_public_ip'] = parsed_args.delete_public_ip
        if parsed_args.notification:
            lst = []
            for notification in parsed_args.notification:
                lst.append(notification)
            args['notifications'] = lst

        client = self.app.client_manager.auto_scaling
        group = client.find_group(parsed_args.group, ignore_missing=False)
        group = client.update_group(group, **args)
        display_columns, columns = _get_columns(group)
        data = utils.get_item_properties(group, columns, formatters={})

        return (display_columns, data)


class DisableAutoScalingGroup(command.Command):
    _description = _('Disable/pause AutoScalinig group')

    def get_parser(self, prog_name):
        parser = super(DisableAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('ID or name of the configuration group')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling
        if parsed_args.group:
            group = client.find_group(parsed_args.group, ignore_missing=False)
            self.app.client_manager.auto_scaling.pause_group(group)


class EnableAutoScalingGroup(command.Command):
    _description = _('Enable/resume AutoScalinig group')

    def get_parser(self, prog_name):
        parser = super(EnableAutoScalingGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('ID or name of the configuration group')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling
        if parsed_args.group:
            group = client.find_group(parsed_args.group, ignore_missing=False)
            self.app.client_manager.auto_scaling.resume_group(group)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack.dns.v2 import recordset
from otcextensions.sdk import dnsmixin


class Recordset(dnsmixin.DNSProxyMixin, recordset.Recordset):
    """DNS Recordset Resource"""

    _query_mapping = resource.QueryParameters(
        'name', 'type', 'ttl', 'data', 'status', 'description',
        'limit', 'marker', 'offset', 'tags')

    #: Properties
    #: Is the recordset created by system.
    is_default = resource.Body('default', type=bool)
    #: Timestamp when the zone was last updated.
    updated_at = resource.Body('update_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class IdentitySpec(resource.Resource):
    name = resource.Body('name')
    location = resource.Body('location')
    validation = resource.Body('validation')


class CustomAuthorizer(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/authorizers'
    resources_key = 'authorizer_list'
    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_patch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'id', 'type', 'name')
    gateway_id = resource.URI('gateway_id')
    name = resource.Body('name')
    type = resource.Body('type')
    authorizer_type = resource.Body('authorizer_type')
    authorizer_uri = resource.Body('authorizer_uri')
    network_type = resource.Body('network_type')
    authorizer_version = resource.Body('authorizer_version')
    authorizer_alias_uri = resource.Body('authorizer_alias_uri')
    identities = resource.Body('identities',
                               type=list,
                               list_type=IdentitySpec)
    ttl = resource.Body('ttl')
    user_data = resource.Body('user_data')
    ld_api_id = resource.Body('ld_api_id')
    need_body = resource.Body('need_body', type=bool)
    id = resource.Body('id')
    create_time = resource.Body('create_time')
    roma_app_id = resource.Body('roma_app_id')
    roma_app_name = resource.Body('roma_app_name')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json
import uuid

from datetime import datetime

from openstackclient.tests.functional import base


class VpcTestCase(base.TestCase):
    """Common functional test bits for VPC commands"""

    CURR_TIME = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")

    def setUp(self):
        super(VpcTestCase, self).setUp()
        UUID = uuid.uuid4().hex[:8]
        self.LOCAL_ROUTER_NAME = 'test-local-router-otce-cli' + UUID
        self.PEER_ROUTER_NAME = 'test-peer-router-otce-cli' + UUID
        self.PEERING_NAME = 'test-peering-otce-cli-' + UUID

        self.LOCAL_ROUTER_ID = None
        self.PEER_ROUTER_ID = None
        self.PEERING_ID = None

    def create_vpc_peering(self, name=None):
        self._create_routers()
        name = name or self.PEERING_NAME
        json_output = json.loads(self.openstack(
            'vpc peering create '
            '{name} '
            '--local-router-id "{local_router_id}" '
            '--peer-router-id "{peer_router_id}" '
            '-f json'.format(
                name=name,
                local_router_id=self.LOCAL_ROUTER_ID,
                peer_router_id=self.PEER_ROUTER_ID)
        ))
        self.assertIsNotNone(json_output)
        self.PEERING_ID = json_output['id']
        return json_output

    def delete_vpc_peering(self):
        self.addCleanup(self._delete_routers)
        self.openstack('vpc peering delete {}'.format(self.PEERING_ID))

    def _create_routers(self):
        local_router = json.loads(self.openstack(
            'router create -f json ' + self.LOCAL_ROUTER_NAME
        ))
        self.LOCAL_ROUTER_ID = local_router['id']

        peer_router = json.loads(self.openstack(
            'router create -f json ' + self.PEER_ROUTER_NAME
        ))
        self.PEER_ROUTER_ID = peer_router['id']

    def _delete_routers(self):
        self.openstack(
            'router delete {} {}'.format(
                self.LOCAL_ROUTER_ID, self.PEER_ROUTER_ID
            ))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import resource
from openstack import utils

from otcextensions.sdk import sdk_resource

_logger = _log.setup_logging('openstack')


class Instance(sdk_resource.Resource):

    base_path = '/instances'
    resource_key = 'instance'
    resources_key = 'instances'
    # service_expectes_json_type = True

    # capabilities
    allow_create = True
    allow_delete = True
    allow_get = True
    allow_update = True
    allow_list = True

    # project_id = resource.URI('project_id')
    # Properties
    #: Instance id
    id = resource.Body('id')
    #: Instance status
    status = resource.Body('status')
    #: Instance name
    name = resource.Body('name')
    #: Links
    #: *Type:list*
    # links = resource.Body('links', type=list)
    #: Flavor information
    #: *Type: dict*
    flavor = resource.Body('flavor', type=dict)
    #: Data store information
    #: *Type: dict*
    datastore = resource.Body('datastore', alias='datastore_info', type=dict)
    datastore_info = resource.Body('dataStoreInfo', type=dict)
    # datastore_info = resource.Body('dataStoreInfo', type=dict)
    #: Region
    region = resource.Body('region')
    #: Tenant_id
    #: *Type: str*
    tenante_id = resource.Body('tenant_id')
    #: Volume information
    #: *Type: dict*
    volume = resource.Body('volume', type=dict)
    #: Host name of the instance
    #: *Type:str*
    hostname = resource.Body('hostname')
    #: Instance ip
    #: *Type:str*
    created = resource.Body('ip')
    #: Instance created time
    #: *Type:str*
    created = resource.Body('created')
    #: Instance updated time
    #: *Type:str*
    updated = resource.Body('updated')
    #: Fault, only validate if fault
    #: *Type:dict*
    fault = resource.Body('fault', type=dict)
    #: Replicas
    #: *Type:dict*
    replicas = resource.Body('replicas', type=list)
    #: Configuration
    #: *Type:dict*
    configuration = resource.Body('configuration', type=dict)
    #: Locality
    #: Not supported in RDS
    #: *Type:str*
    locality = resource.Body('locality')
    #: Used local Storage
    #: Not supported in RDS
    local_storage_used = resource.Body('local_storage_used')
    #: Password
    #: The password of the database root user(i.e. the administrative user).
    password = resource.Body('password')
    #: Cluster id
    cluster_id = resource.Body('cluster_id')
    #: Shard id
    shard_id = resource.Body('shard_id')
    #: Server id
    #: The ID of the underlying Nova instance for an instance.
    #: *Type:str*
    server_id = resource.Body('server_id')
    #: Volume id
    #: The ID of a volume.
    #: *Type:str*
    volume_id = resource.Body('volume_id')

    port = resource.Body('dbPort', type=int)

    slave_id = resource.Body('slaveId')
    #: encrypted_rpc_messaging
    #: Whether the instance is using encrypted rpm messaging feature or not.
    #: *Type:bool*
    encrypted_rpc_messaging = resource.Body('encrypted_rpc_messaging')

    # Additional Creation properties
    #: Users
    #: *Type:list*
    users = resource.Body('users', type=list)
    #: Flavor ID
    #: *Type:uuid*
    flavorRef = resource.Body('flavorRef')
    #: Modules
    #: *Type:dict*
    modules = resource.Body('modules', type=dict)
    #: Restore point
    #: *Type:dict*
    restore_point = resource.Body('restore_point', type=dict)
    #: Availability Zone
    #: The availability zone of the instance.
    #: *Type:str*
    availability_zone = resource.Body('availabilityZone', alias='azcode')
    #: Nics interface list
    #: *Type:dict*
    nics = resource.Body('nics', type=dict)
    #: Id of the master
    #: *Type:str*
    replica_of = resource.Body('replica_of')
    #: Replica of the instance
    #: *Type:int*
    replica_count = resource.Body('replica_count')
    #: region name
    #: *Type:str*
    region_name = resource.Body('region_name')
    #: Databases object
    #: *Type:list*
    databases = resource.Body('databases', type=list)

    # RDS additinal attributes
    #: Instance type readreplica/master/slave
    type = resource.Body('type')
    #: Private cloud id
    vpc = resource.Body('vpc')
    #: Security group
    #: *Type: dict*
    security_group = resource.Body('securityGroup', type=dict)
    #: Backup Strategy
    #: *Type: dict*
    backup_strategy = resource.Body('backupStrategy', type=dict)
    #: HA information
    #: *Type: dict*
    ha = resource.Body('ha', type=dict)
    #: Restore Point, create new instance from restore
    #: *Type: dict*
    restore_point = resource.Body("restorePoint", type=dict)
    #: Root password
    dbRtPd = resource.Body("dbRtPd")
    #: Status of configuration
    configurationStatus = resource.Body("configurationStatus")
    #: Id of configuration
    paramsGroupId = resource.Body('paramsGroupId')
    #: Id of subnet
    subnetid = resource.Body('subnetid')
    #: Instance role
    role = resource.Body('role')
    #: Internal subnet id
    internalSubnetId = resource.Body('internalSubnetId')
    #: Group of instance
    group = resource.Body('group')
    #: Secure group id
    securegroup = resource.Body('securegroup')
    #: Az code
    azcode = resource.Body('azcode')
    #: DB user
    dbuser = resource.Body('dbuser')
    #: Storage Engine
    storeEngine = resource.Body('storeEngine')
    #: Pay model
    payModel = resource.Body('payModel')
    #: Slave of instance
    slave_of = resource.Body('slave_of')
    # Indicates the EIP for public access, including
    # the IP address and port number.
    # *Type:string*
    publicEndpoint = resource.Body('publicEndpoint')

    def _action(self, exec_method, json):
        """Executes the action

        :returns: ``None``
        """
        base_url = self.base_path % self._uri.attributes
        url = utils.urljoin(base_url, self.id, 'action')
        # NOTE: due to the API stupidity we need to send also X-Language
        return exec_method(
            url,
            json=json,
            headers={'X-Language': 'en-us'}
        )

    def restart(self, session):
        """Restart the database instance

        :returns: ``None``
        """
        body = {'restart': {}}
        return self._action(session.post, body)

    def resize(self, session, flavor_reference):
        """Resize the database instance flavor

        :returns: ``None``
        """
        body = {'resize': {'flavorRef': flavor_reference}}
        return self._action(session.post, body)

    def resize_volume(self, session, volume_size):
        """Resize the volume attached to the instance

        :returns: ``None``
        """
        body = {'resize': {'volume': volume_size}}
        return self._action(session.post, body)

    def restore(self, session, backupRef):
        """Restores database to the given backup rference

        :returns: ``None``
        """
        body = {"restore": {"backupRef": backupRef}}
        return self._action(session.post, body)

        # TODO(agoncharov) call returns jobId



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print(monitors):
    for monitor in monitors:
        monitor.cpu_core = f"{monitor.cpu_core_usage}/{monitor.cpu_core_total}"
        monitor.cpu_memory = (
            f"{monitor.cpu_memory_usage}/{monitor.cpu_memory_total}"
        )
        monitor.gpu = f"{monitor.gpu_usage}/{monitor.gpu_total}"
        yield monitor


class ServiceMonitor(command.Lister):
    _description = _("Get Monitoring Information of a Service.")
    columns = (
        "Model Id",
        "Model Name",
        "Model Version",
        "Invocation Times",
        "Failed Times",
        "CPU Core",
        "CPU Memory",
        "GPU",
    )

    def get_parser(self, prog_name):
        parser = super(ServiceMonitor, self).get_parser(prog_name)

        parser.add_argument(
            "service",
            metavar="<service>",
            help=_("Service ID or Name."),
        )
        parser.add_argument(
            "--node-id",
            metavar="<node_id>",
            help=_(
                "ID of the node to be queried. By default, "
                "all nodes are queried."
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        query_params = {}
        if parsed_args.node_id:
            query_params["node_id"] = parsed_args.node_id

        service = client.find_service(
            parsed_args.service, ignore_missing=False
        )
        data = client.service_monitor(service.id, **query_params)

        if data:
            data = set_attributes_for_print(data)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.deh.v1 import host_type


FAKE_ID = '68d5745e-6af2-40e4-945d-fe449be00148'
EXAMPLE = {
    'host_type': 'general',
    'host_type_name': 'general computing'
}


class TestHost(base.TestCase):

    def test_basic(self):
        sot = host_type.HostType()

        self.assertEqual('/availability-zone/%(availability_zone)s/'
                         'dedicated-host-types',
                         sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):

        sot = host_type.HostType(**EXAMPLE)
        self.assertEqual(EXAMPLE['host_type'], sot.host_type)
        self.assertEqual(EXAMPLE['host_type_name'], sot.host_type_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Signature(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/signs'

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    resources_key = 'signs'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'name',
        'id', 'precise_search'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Signature key name. It can contain letters,
    # digits, and underscores(_) and must start with a letter.
    name = resource.Body('name', type=str)
    # Signature key type.
    # hmac
    # basic
    # public_key
    # aes
    sign_type = resource.Body('sign_type', type=str)
    # Signature key.
    sign_key = resource.Body('sign_key', type=str)
    # Signature secret.
    sign_secret = resource.Body('sign_secret', type=str)
    # Signature algorithm. Specify a signature algorithm only
    # when using an AES signature key. By default, no algorithm is used.
    # Enumeration values:
    # aes-128-cfb
    # aes-256-cfb
    sign_algorithm = resource.Body('sign_algorithm', type=str)

    # Attributes
    # Update time.
    updated_at = resource.Body('update_time', type=str)
    # Creation time.
    created_at = resource.Body('create_time', type=str)
    # Number of bound APIs.
    bind_num = resource.Body('bind_num', type=int)
    # Number of custom backends bound.
    ldapi_bind_num = resource.Body('ldapi_bind_num', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Vpc(resource.Resource):
    # Proxy host.
    vpc_channel_proxy_host = resource.Body('vpc_channel_proxy_host', type=str)
    # VPC channel ID.
    vpc_channel_id = resource.Body('vpc_channel_id', type=str)

    # Attributes
    # Cloud server ID.
    ecs_id = resource.Body('ecs_id', type=str)
    # Cloud server name.
    ecs_name = resource.Body('ecs_name', type=str)
    # Indicates whether to use the cascading mode.
    cascade_flag = resource.Body('cascade_flag', type=bool)
    # VPC channel port.
    vpc_channel_port = resource.Body('vpc_channel_port', type=int)


class BackendParam(resource.Resource):
    # Parameter type.
    # REQUEST: backend parameter
    # CONSTANT: constant parameter
    # SYSTEM: system parameter
    origin = resource.Body('origin', type=str)
    # Parameter name.
    name = resource.Body('name', type=str)
    # Description, which can contain a maximum of 255 characters.
    remark = resource.Body('remark', type=str)
    # Parameter location. The value can be PATH, QUERY, or HEADER.
    location = resource.Body('location', type=str)
    # Parameter value, which can contain a maximum of 255 characters.
    # The gateway parameters are as follows:
    # $context.sourceIp: source IP address of the API caller.
    # $context.stage: deployment environment in which the API is called.
    # $context.apiId: API ID.
    # $context.appId: ID of the app used by the API caller.
    # $context.requestId: request ID generated when the API is called.
    # $context.serverAddr: address of the gateway server.
    # $context.serverName: name of the gateway server.
    # $context.handleTime: time when the API request is processed.

    # $context.providerAppId: ID of the app used by the API owner.
    # This parameter is currently not supported.

    # Frontend authentication parameter:
    # prefixed with "$context.authorizer.frontend.".
    # For example, to return "aaa" upon successful custom authentication,
    # set this parameter to "$context.authorizer.frontend.aaa".
    #
    # Backend authentication parameter:
    # prefixed with "$context.authorizer.backend.".
    # For example, to return "aaa" upon successful custom authentication,
    # set this parameter to "$context.authorizer.backend.aaa".
    value = resource.Body('value', type=str)

    # Attributes
    # ID
    id = resource.Body('id', type=str)
    # Request parameter ID.
    req_param_id = resource.Body('req_param_id', type=str)


class ApiMock(resource.Resource):
    # Description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # Response
    result_content = resource.Body('result_content', type=str)
    # Function version. It cannot exceed 64 characters.
    version = resource.Body('version', type=str)
    # Backend custom authorizer ID.
    authorizer_id = resource.Body('authorizer_id', type=str)

    # Attributes
    # ID
    id = resource.Body('id', type=str)
    # Backend service status.
    status = resource.Body('status', type=int)
    # Registration time.
    register_time = resource.Body('register_time', type=str)
    # Update time.
    update_time = resource.Body('update_time', type=str)


class ApiFunc(resource.Resource):
    # Function URN.
    function_urn = resource.Body('function_urn', type=str)
    # Description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # Invocation mode.
    # async: asynchronous
    # sync: synchronous
    invocation_type = resource.Body('invocation_type', type=str)
    # Function network architecture.
    # V1: non-VPC
    # V2: VPC
    network_type = resource.Body('network_type', type=str)
    # Function version.
    # If both a function alias URN and version are passed,
    # only the alias URN will be used.
    version = resource.Body('version', type=str)
    # Function alias URN.
    alias_urn = resource.Body('alias_urn', type=str)
    # Timeout allowed for APIG to request the backend service.
    timeout = resource.Body('timeout', type=int)
    # Backend custom authorizer ID.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # Backend request protocol of a function.
    # The value can be HTTPS (default) or
    # GRPCS (available when the frontend request protocol is GRPCS).
    # Default: HTTPS
    req_protocol = resource.Body('req_protocol', type=str)

    # Attributes
    # ID
    id = resource.Body('id', type=str)
    # Registration time.
    register_time = resource.Body('register_time', type=str)
    # status
    status = resource.Body('status', type=int)
    # Update time.
    update_time = resource.Body('update_time', type=str)


class ReqParam(resource.Resource):
    # Parameter name.
    name = resource.Body('name', type=str)
    # Parameter type.
    # Enumeration values:
    # STRING
    # NUMBER
    type = resource.Body('type', type=str)
    # Parameter location.
    # Enumeration values:
    # PATH
    # QUERY
    # HEADER
    location = resource.Body('location', type=str)
    # Default value.
    default_value = resource.Body('default_value', type=str)
    # Example value.
    sample_value = resource.Body('sample_value', type=str)
    # Indicates whether the parameter is required. 1: yes 2: no
    required = resource.Body('required', type=int)
    # Indicates whether validity check is enabled.
    # 1: enabled
    # 2: disabled
    valid_enable = resource.Body('valid_enable', type=int)
    # Description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # Enumerated value.
    enumerations = resource.Body('enumerations', type=str)
    # Minimum value.
    min_num = resource.Body('min_num', type=int)
    # Maximum value.
    max_num = resource.Body('max_num', type=int)
    # Minimum length.
    min_size = resource.Body('min_size', type=int)
    # Maximum length.
    max_size = resource.Body('max_size', type=int)
    # Regular expression validation rule.
    regular = resource.Body('regular', type=str)
    # JSON validation rule.
    json_schema = resource.Body('json_schema', type=str)
    # Indicates whether to transparently transfer the parameter. 1: yes 2: no
    pass_through = resource.Body('pass_through', type=int)

    # Attributes
    # ID
    id = resource.Body('id', type=str)


class AuthOpt(resource.Resource):
    # Indicates whether AppCode authentication is enabled.
    # This parameter is valid only if auth_type is set to App.
    # The default value is DISABLE.
    # DISABLE: AppCode authentication is disabled.
    # HEADER: AppCode authentication is enabled and the AppCode
    # is located in the header.
    # Default: DISABLE
    app_code_auth_type = resource.Body('app_code_auth_type', type=str)


class ApiCondition(resource.Resource):
    # Input parameter name.
    # This parameter is required if the policy type is param.
    req_param_name = resource.Body('req_param_name', type=str)
    # Name of a built-in gateway parameter.
    # This parameter is required if the policy type is system.
    # The following parameters are supported:
    # req_path: request path. For example, /a/b.
    # req_method: request method. For example, GET.
    # reqPath: request path, which is discarded. For example, /a/b.
    # reqMethod: request method, which is discarded. For example, GET.
    sys_param_name = resource.Body('sys_param_name', type=str)
    # COOKIE parameter name.
    # This parameter is required if the policy type is cookie.
    cookie_param_name = resource.Body('cookie_param_name', type=str)
    # System parameter: frontend authentication parameter name.
    frontend_authorizer_param_name = resource.Body(
        'frontend_authorizer_param_name', type=str
    )
    # Policy condition.
    # exact: Exact match
    # enum: Enumeration
    # pattern: Regular expression
    condition_type = resource.Body('condition_type', type=str)
    # Policy type
    # param: parameter
    # source: source IP address
    # system: system parameter - gateway built-in parameter
    # cookie: COOKIE parameter
    # frontend_authorizer: system parameter - frontend authentication parameter
    condition_origin = resource.Body('condition_origin', type=str)
    # Policy value.
    # This parameter is required when the policy type is param,
    # source, cookie, or frontend_authorizer.
    condition_value = resource.Body('condition_value', type=str)

    # Attributes
    # ID.
    id = resource.Body('id', type=str)
    # Input parameter ID.
    req_param_id = resource.Body('req_param_id', type=str)
    # Input parameter location.
    req_param_location = resource.Body('req_param_location', type=str)


class ApiPolicyMock(resource.Resource):
    # Response.
    result_content = resource.Body('result_content', type=str)
    # Effective mode of the backend policy.
    # ALL: All conditions are met.
    # ANY: Any condition is met.
    effect_mode = resource.Body('effect_mode', type=str)
    # Backend name.
    # It must start with a letter and can contain letters,
    # digits, and underscores (_).
    name = resource.Body('name', type=str)
    # Backend parameters. This is unavailable for the GRPC backend.
    backend_params = resource.Body(
        'backend_params', type=list, list_type=BackendParam
    )
    # Policy conditions.
    conditions = resource.Body(
        'conditions', type=list, list_type=ApiCondition
    )
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)

    # Attributes
    # ID.
    id = resource.Body('id', type=str)


class ApiPolicyFunction(resource.Resource):
    # Function URN.
    function_urn = resource.Body('function_urn', type=str)
    # Invocation mode.
    # async: asynchronous
    # sync: synchronous
    invocation_type = resource.Body('invocation_type', type=str)
    # Function network architecture.
    # V1: non-VPC
    # V2: VPC
    network_type = resource.Body('network_type', type=str)
    # Function version.
    version = resource.Body('version', type=str)
    # Function alias URN.
    alias_urn = resource.Body('alias_urn', type=str)
    # Timeout allowed for APIG to request the backend service.
    timeout = resource.Body('timeout', type=int)
    # Backend request protocol of a function.
    # The value can be HTTPS (default) or
    # GRPCS (available when the frontend request protocol is GRPCS).
    req_protocol = resource.Body('req_protocol', type=str)
    # Effective mode of the backend policy.
    # ALL: All conditions are met.
    # ANY: Any condition is met.
    effect_mode = resource.Body('effect_mode', type=str)
    # Backend name.
    # It must start with a letter and can contain letters,
    # digits, and underscores (_).
    name = resource.Body('name', type=str)
    # Backend parameters. This is unavailable for the GRPC backend.
    backend_params = resource.Body(
        'backend_params', type=list, list_type=BackendParam
    )
    # Policy conditions.
    conditions = resource.Body(
        'conditions', type=list, list_type=ApiCondition
    )
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)


class BackendApi(resource.Resource):
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # Backend service address.
    # It must be in the format "Host name:Port number",
    # for example, apig.example.com:7443
    url_domain = resource.Body('url_domain', type=str)
    # Request protocol. You can select GRPCS for the GRPC backend.
    req_protocol = resource.Body('req_protocol', type=str)
    # Description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # Request method. For the GRPC backend,
    # the request method is fixed to POST.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # Web backend version, which can contain a maximum of 16 characters.
    version = resource.Body('version', type=str)
    # Request address, which can contain request parameters
    # enclosed with braces ({})
    req_uri = resource.Body('req_uri', type=str)
    # Timeout allowed for APIG to request the backend service.
    timeout = resource.Body('timeout', type=int)
    # Indicates whether to enable two-way authentication.
    enable_client_ssl = resource.Body('enable_client_ssl', type=bool)
    # Number of retry attempts to request the backend service.
    # The default value is -1. The value ranges from -1 to 10.
    retry_count = resource.Body('retry_count', type=str)
    # VPC channel details.
    # This parameter is required if vpc_channel_status is set to 1.
    vpc_channel_info = resource.Body('vpc_channel_info', type=Vpc)
    # Indicates whether to use a VPC channel.
    # 1: A VPC channel is used.
    # 2: No VPC channel is used.
    vpc_channel_status = resource.Body('vpc_channel_status', type=int)

    # Attributes
    # ID.
    id = resource.Body('id', type=str)
    # Backend service status.
    status = resource.Body('status', type=int)
    # Registration time.
    register_time = resource.Body('register_time', type=str)
    # Update time.
    update_time = resource.Body('update_time', type=str)


class ApiPolicyHttp(resource.Resource):
    # Endpoint of the policy backend.
    # It must be in the format "Domain name:Port number",
    # for example, apig.example.com:7443.
    # If the port number is not specified,
    # the default HTTPS port 443 or the default HTTP port 80 is used.
    url_domain = resource.Body('url_domain', type=str)
    # Request protocol. Options include HTTP, HTTPS, and GRPCS.
    # You can select GRPCS for a GRPC backend.
    req_protocol = resource.Body('req_protocol', type=str)
    # Request method. Options include GET, POST, PUT, DELETE,
    # HEAD, PATCH, OPTIONS, and ANY.
    # This is fixed to POST for the GRPC backend.
    req_method = resource.Body('req_method', type=str)
    # Request address, which can contain request
    # parameters enclosed with braces ({}).
    req_uri = resource.Body('req_uri', type=str)
    # Timeout allowed for APIG to request the backend service.
    timeout = resource.Body('timeout', type=int)
    # Number of retry attempts to request the backend service.
    # The default value is -1. The value ranges from -1 to 10.
    retry_count = resource.Body('retry_count', type=str)
    # Effective mode of the backend policy.
    # ALL: All conditions are met.
    # ANY: Any condition is met.
    effect_mode = resource.Body('effect_mode', type=str)
    # Backend name. It must start with a letter and can contain letters,
    # digits, and underscores (_).
    name = resource.Body('name', type=str)
    # Backend parameters. This is unavailable for the GRPC backend.
    backend_params = resource.Body(
        'backend_params', type=list, list_type=BackendParam
    )
    # Policy conditions.
    conditions = resource.Body(
        'conditions', type=list, list_type=ApiCondition
    )
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # VPC channel details.
    # This parameter is required if vpc_channel_status is set to 1.
    vpc_channel_info = resource.Body('vpc_channel_info', type=Vpc)
    # Indicates whether to use a VPC channel.
    # 1: A VPC channel is used.
    # 2: No VPC channel is used.
    vpc_channel_status = resource.Body('vpc_channel_status', type=int)

    # Attributes
    # ID.
    id = resource.Body('id', type=str)


class UrlDomain(resource.Resource):
    # Domain ID.
    id = resource.Body('id', type=str)
    # Domain name.
    name = resource.Body('domain', type=str)
    # CNAME resolution status of the domain name.
    # 1: not resolved
    # 2: resolving
    # 3: resolved
    # 4: resolution failed
    cname_status = resource.Body('cname_status', type=int)
    # SSL certificate ID.
    ssl_id = resource.Body('ssl_id', type=str)
    # SSL certificate name.
    ssl_name = resource.Body('ssl_name', type=str)
    # Minimum SSL version. TLS 1.1 and TLS 1.2 are supported.
    min_ssl_version = resource.Body('min_ssl_version', type=str)
    # Whether to enable client certificate verification.
    # This parameter is available only when a certificate is bound
    verified_client_certificate_enabled = resource.Body(
        'verified_client_certificate_enabled', type=bool
    )
    # Independent domain names bound to the API group.
    is_has_trusted_root_ca = resource.Body('is_has_trusted_root_ca', type=bool)


class ApiGroupCommonInfo(resource.Resource):
    # ID.
    id = resource.Body('id', type=str)
    # API group name.
    name = resource.Body('name', type=str)
    # Status. 1: valid
    status = resource.Body('status', type=int)
    # Subdomain name that APIG automatically allocates to the API group.
    sl_domain = resource.Body('sl_domain', type=str)
    # Creation time.
    register_time = resource.Body('register_time', type=str)
    # Last modification time.
    update_time = resource.Body('update_time', type=str)
    # Indicates whether the API group has been listed on KooGallery.
    # 1: listed
    # 2: not listed
    # 3: under review
    on_sell_status = resource.Body('on_sell_status', type=int)
    # Independent domain names bound to the API group.
    url_domains = resource.Body('url_domains', type=list, list_type=UrlDomain)
    sl_domain_access_enabled = resource.Body(
        'sl_domain_access_enabled', type=bool
    )


class Api(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis'

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    resources_key = 'apis'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'name',
        'id', 'group_id', 'req_protocol', 'req_method',
        'req_uri', 'auth_type', 'env_id', 'type', 'precise_search',
        'vpc_channel_name', 'return_data_mode'
    )

    # Properties
    gateway_id = resource.URI('gateway_id', type=str)
    # API name.
    name = resource.Body('name', type=str)
    # API type.
    # 1: public API
    # 2: private API
    type = resource.Body('type', type=int)
    # API version.
    version = resource.Body('version', type=str)
    # API request protocol:
    # HTTP
    # HTTPS
    # BOTH: Both HTTP and HTTPS are supported.
    # GRPCS
    req_protocol = resource.Body('req_protocol', type=str)
    # API request method. If the request protocol is set to GRPC,
    # the request method is fixed to POST.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # Request address, which can contain request
    # parameters enclosed with braces ({})
    req_uri = resource.Body('req_uri', type=str)
    # API authentication mode. Options:
    # NONE
    # APP
    # IAM
    # AUTHORIZER: custom authentication.
    # When auth_type is set to AUTHORIZER,
    # the authorizer_id field is mandatory.
    auth_type = resource.Body('auth_type', type=str)
    # Security authentication parameter.
    auth_opt = resource.Body('auth_opt', type=AuthOpt)
    # Indicates whether CORS is supported.
    # TRUE: supported
    # FALSE: not supported
    cors = resource.Body('cors', type=bool)
    # API matching mode:
    # SWA: Prefix match
    # NORMAL: Exact match Default value: NORMAL
    match_mode = resource.Body('match_mode', type=str)
    # Backend type. Options:
    # HTTP: web backend
    # FUNCTION: FunctionGraph backend. When backend_type is set to FUNCTION,
    # the func_info field is mandatory.
    # MOCK: mock backend. When backend_type is set to MOCK,
    # the mock_info field is mandatory.
    # GRPC: gRPC backend.
    backend_type = resource.Body('backend_type', type=str)
    # API description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API request body, which can be an example request body,
    # media type, or parameters.
    # Ensure that the request body does not exceed 20,480 characters.
    body_remark = resource.Body('body_remark', type=str)
    # Example response for a successful request.
    # Ensure that the response does not exceed 20,480 characters.
    # This is unavailable if the request protocol is set to GRPC.
    result_normal_sample = resource.Body('result_normal_sample', type=str)
    # Example response for a failed request.
    # Ensure that the response does not exceed 20,480 characters.
    # This is unavailable if the request protocol is set to GRPC.
    result_failure_sample = resource.Body('result_failure_sample', type=str)
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # Tag.
    # Use letters, digits, and special characters (-*#%.:_)
    # and start with a letter.
    tags = resource.Body('tags', type=list)
    # Group response ID.
    response_id = resource.Body('response_id', type=str)
    # Request content type:
    # application/json
    # application/xml
    # multipart/form-data
    # text/plain
    content_type = resource.Body('content_type', type=str)
    # Mock backend details.
    mock_info = resource.Body('mock_info', type=ApiMock)
    # FunctionGraph backend details.
    func_info = resource.Body('func_info', type=ApiFunc)
    # Request parameters.
    # This is unavailable if the request protocol is set to GRPC.
    req_params = resource.Body(
        'req_params', type=list, list_type=ReqParam
    )
    # Backend parameters.
    # This is unavailable if the request protocol is set to GRPC.
    backend_params = resource.Body(
        'backend_params', type=list, list_type=BackendParam
    )
    # Mock backend policies.
    policy_mocks = resource.Body(
        'policy_mocks', type=list, list_type=ApiPolicyMock
    )
    # FunctionGraph backend policies.
    policy_functions = resource.Body(
        'policy_functions', type=list, list_type=ApiPolicyFunction
    )
    # Web backend details.
    backend_api = resource.Body('backend_api', type=BackendApi)
    # Web backend policies.
    policy_https = resource.Body(
        'policy_https', type=list, list_type=ApiPolicyHttp
    )

    # Attributes
    # App status.
    status = resource.Body('status', type=int)
    # Indicates whether to enable orchestration.
    arrange_necessary = resource.Body('arrange_necessary', type=int)
    # Time when the API is registered.
    register_time = resource.Body('register_time', type=str)
    # Time when the API was last modified.
    update_time = resource.Body('update_time', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Version of the API group to which the API belongs.
    group_version = resource.Body('group_version', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # Publication time.
    publish_time = resource.Body('publish_time', type=str)
    # API group information.
    api_group_info = resource.Body('api_group_info', type=ApiGroupCommonInfo)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack.tests.unit import base
from otcextensions.sdk.dws.v1 import flavor

EXAMPLE = {
    'id': uuid.uuid4().hex,
    'detail': [
        {'value': '32', 'type': 'vCPU'},
        {'value': '4000', 'type': 'SSD', 'unit': 'GB'},
        {'value': '256', 'type': 'mem', 'unit': 'GB'},
        {'value': 'eu-de-02,eu-de-01', 'type': 'availableZones'},
    ],
    'spec_name': 'dws2.m6.8xlarge.8',
    'vCPU': '32',
    'disk_type': 'SSD',
    'disk_size': '4000',
    'mem': '256',
    'availableZones': 'eu-de-02,eu-de-01',
}


class TestFlavor(base.TestCase):
    def setUp(self):
        super(TestFlavor, self).setUp()

    def test_basic(self):
        sot = flavor.Flavor()

        self.assertEqual('/node-types', sot.base_path)
        self.assertEqual('node_types', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = flavor.Flavor(**EXAMPLE)
        self.assertEqual(EXAMPLE['availableZones'], sot.availability_zones)
        self.assertEqual(EXAMPLE['detail'], sot.detail)
        self.assertEqual(EXAMPLE['disk_type'], sot.disk_type)
        self.assertEqual(int(EXAMPLE['disk_size']), sot.disk_size)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(int(EXAMPLE['mem']), sot.ram)
        self.assertEqual(EXAMPLE['spec_name'], sot.name)
        self.assertEqual(EXAMPLE['spec_name'], sot.spec_name)
        self.assertEqual(int(EXAMPLE['vCPU']), sot.vcpu)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions
from otcextensions.sdk.apig.v2.api import ApiFunc
from otcextensions.sdk.apig.v2.api import ApiMock
from otcextensions.sdk.apig.v2.api import ApiGroupCommonInfo
from otcextensions.sdk.apig.v2.api import BackendApi
from otcextensions.sdk.apig.v2.api import ReqParam
from otcextensions.sdk.apig.v2.api import BackendParam
from otcextensions.sdk.apig.v2.api import ApiPolicyMock
from otcextensions.sdk.apig.v2.api import ApiPolicyFunction
from otcextensions.sdk.apig.v2.api import ApiPolicyHttp


class PublishApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/action'

    allow_create = True

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Operation to perform.
    # online: publish APIs
    # offline: take APIs offline
    action = resource.Body('action', type=int)
    # ID of the environment in which the API will be published.
    env_id = resource.Body('env_id', type=str)
    # ID of the API to be published or taken offline.
    api_id = resource.Body('api_id', type=str)
    # Description about the publishing. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)

    # Attributes
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # API name.
    api_name = resource.Body('api_name', type=str)
    # Publication time.
    publish_time = resource.Body('publish_time', type=str)
    # API version currently in use.
    version_id = resource.Body('version_id', type=str)

    def _action(self, session, body):
        """Preform actions given the message body.
        """
        url = self.base_path % {'gateway_id': body['gateway_id']}
        response = session.post(url, json=body)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def publish_api(self, session, **attrs):
        attrs['action'] = "online"
        return self._action(session, attrs)

    def take_api_offline(self, session, **attrs):
        attrs['action'] = "offline"
        return self._action(session, attrs)


class CheckApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/check'

    allow_create = True

    # Properties
    gateway_id = resource.URI('gateway_id')
    # API name.
    name = resource.Body('name')
    # Request method.
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method')
    # Access address of the API.
    req_uri = resource.Body('req_uri')
    # API matching mode:
    # SWA: Prefix match
    # NORMAL: Exact match
    match_mode = resource.Body('match_mode')
    # Group ID.
    group_id = resource.Body('group_id')
    # Integration application ID.
    roma_app_id = resource.Body('roma_app_id')
    # ID of the API to be compared.
    api_id = resource.Body('api_id')
    # Verification type:
    # path: path type
    # name: name type
    type = resource.Body('type')


class DebugApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/debug/%(api_id)s'

    allow_create = True

    # Properties
    gateway_id = resource.URI('gateway_id')
    api_id = resource.URI('api_id')
    # Message body, with a maximum of 2,097,152 bytes.
    body = resource.Body('body')
    # Header parameters, with each value being a character string array.
    header = resource.Body('header', type=dict)
    # API request method.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    method = resource.Body('method', type=str)
    # Debugging mode:
    # DEVELOPER: Debug the definitions of an API that has not been published.
    # CONSUMER: Debug the definitions of an API that has been
    # published in a specified environment.
    mode = resource.Body('mode', type=str)
    # Request path of the API, starting with a slash (/)
    # and containing up to 1024 characters.
    path = resource.Body('path', type=str)
    # Query strings, with each value being a character string array.
    query = resource.Body('query', type=dict)
    # Request protocol.
    # HTTP
    # HTTPS
    scheme = resource.Body('scheme', type=str)
    # AppKey used in the debugging request.
    app_key = resource.Body('app_key', type=str)
    # AppSecret used in the debugging request.
    app_secret = resource.Body('app_secret', type=str)
    # Access domain name of the API.
    # If no value is specified, one of the following default
    # values will be used based on the mode:
    # DEVELOPER: The subdomain name of the API group will be used.
    # MARKET: This parameter is not used currently.
    # CONSUMER: The subdomain name of the API group will be used.
    domain = resource.Body('domain', type=str)
    # Running environment specified by the debugging request.
    # This parameter is valid only when mode is set to CONSUMER.
    # If this parameter is not specified, the following default value is used:
    # CONSUMER RELEASE
    stage = resource.Body('stage', type=str)

    # Attributes
    # Body of the debugging request.
    request = resource.Body('request', type=str)
    # Body of the debugging response, with a maximum of 2,097,152 bytes.
    # Any content beyond this threshold will be truncated.
    response = resource.Body('response', type=str)
    # Debugging duration in milliseconds.
    latency = resource.Body('latency', type=int)
    # Debugging logs.
    log = resource.Body('log', type=str)


class PublishApis(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/publish'

    resources_key = 'api_versions'

    allow_create = True
    allow_list = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        "limit", "offset", "env_id", "env_name"
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # API ID.
    api_id = resource.URI('api_id', type=str)
    # Operation to perform.
    # online: publish APIs
    # offline: take APIs offline
    action = resource.Body('action', type=int)
    # ID of the environment in which the API will be published.
    env_id = resource.Body('env_id', type=str)
    # ID of the API to be published or taken offline.
    apis = resource.Body('apis', type=list)
    # API group ID. Either apis or group_id must be specified.
    group_id = resource.Body('group_id', type=str)
    # Description about the publishing. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)

    # Attributes
    # Message for successful API publication or taking offline.
    success = resource.Body('success', type=list)
    # Error message and APIs that fail to be published or taken offline.
    failure = resource.Body('failure', type=list)
    # API version ID.
    version_id = resource.Body('version_id', type=str)
    # API version.
    version_no = resource.Body('version_no', type=str)
    # Name of the environment in which the API has been published.
    env_name = resource.Body('env_name', type=str)
    # Publication time.
    publish_time = resource.Body('publish_time', type=str)
    # Version status.
    # 1: effective
    # 2: not effective
    status = resource.Body('status', type=str)

    def _action(self, session, body, action):
        """Preform actions given the message body.
        """
        url = self.base_path % {'gateway_id': body["gateway_id"]}
        url += f'?action={action}'
        response = session.post(url, json=body)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def publish_apis(self, session, **attrs):
        return self._action(session, attrs, "online")

    def take_apis_offline(self, session, **attrs):
        return self._action(session, attrs, "offline")


class RuntimeDefinitionApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/runtime/%(api_id)s'

    allow_list = True

    _query_mapping = resource.QueryParameters(
        "env_id",
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    api_id = resource.URI('api_id')

    # Attributes
    # API name.
    name = resource.Body('name')
    # Verification type:
    # 1: public API
    # 2: private API
    type = resource.Body('type', type=int)
    # API version.
    version = resource.Body('version', type=str)
    # API request protocol:
    # HTTP
    # HTTPS
    # BOTH: Both HTTP and HTTPS are supported.
    # GRPCS
    # Default: HTTPS
    req_protocol = resource.Body('req_protocol', type=str)
    # API request method. If the request protocol is set to GRPC
    # the request method is fixed to POST.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # Request address, which can contain request parameters enclosed
    # with braces ({}).
    # For example, /getUserInfo/{userId}
    req_uri = resource.Body('req_uri', type=str)
    # API authentication mode. Options:
    # NONE
    # APP
    # IAM
    # AUTHORIZER: custom authentication. When auth_type is set to AUTHORIZER,
    # the authorizer_id field is mandatory.
    auth_type = resource.Body('auth_type', type=str)
    # Security authentication parameter.
    auth_opt = resource.Body('auth_opt', type=dict)
    # Indicates whether CORS is supported.
    # TRUE: supported
    # FALSE: not supported
    cors = resource.Body('cors', type=bool)
    # API matching mode:
    # SWA: Prefix match
    # NORMAL: Exact match Default value: NORMAL
    match_mode = resource.Body('match_mode', type=str)
    # Backend type. Options:
    # HTTP: web backend
    # FUNCTION: FunctionGraph backend. When backend_type is set to FUNCTION,
    # the func_info field is mandatory.
    # MOCK: mock backend. When backend_type is set to MOCK,
    # the mock_info field is mandatory.
    # GRPC: gRPC backend.
    backend_type = resource.Body('backend_type', type=str)
    # API description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API request body
    body_remark = resource.Body('body_remark', type=str)
    # Example response for a successful request.
    # Ensure that the response does not exceed 20,480 characters.
    result_normal_sample = resource.Body('result_normal_sample', type=str)
    # Example response for a failed request.
    # Ensure that the response does not exceed 20,480 characters.
    result_failure_sample = resource.Body('result_failure_sample', type=str)
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # Tags.
    # Use letters, digits, and special characters (-*#%.:_)
    # and start with a letter.
    tags = resource.Body('tags', type=list)
    # Request content type:
    # application/json
    # application/xml
    # multipart/form-data
    # text/plain
    content_type = resource.Body('content_type', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # Subdomain name of the API group.
    sl_domain = resource.Body('sl_domain', type=str)
    # Subdomain names that APIG automatically allocates to the API group.
    sl_domains = resource.Body('sl_domains', type=list)
    # Request parameters.
    req_params = resource.Body('req_params', type=list)


class VersionsApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apis/versions/%(version_id)s'

    allow_list = True
    allow_delete = True

    requires_id = False

    _query_mapping = resource.QueryParameters(
        "env_id",
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    version_id = resource.URI('version_id')

    # Attributes
    # API name.
    name = resource.Body('name')
    # Verification type:
    # 1: public API
    # 2: private API
    type = resource.Body('type', type=int)
    # API version.
    version = resource.Body('version', type=str)
    # API request protocol:
    # HTTP
    # HTTPS
    # BOTH: Both HTTP and HTTPS are supported.
    # GRPCS
    # Default: HTTPS
    req_protocol = resource.Body('req_protocol', type=str)
    # API request method. If the request protocol is set to GRPC
    # the request method is fixed to POST.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # Request address, which can contain request parameters
    # enclosed with braces ({}).
    # For example, /getUserInfo/{userId}
    req_uri = resource.Body('req_uri', type=str)
    # API authentication mode. Options:
    # NONE
    # APP
    # IAM
    # AUTHORIZER: custom authentication. When auth_type is set to AUTHORIZER,
    # the authorizer_id field is mandatory.
    auth_type = resource.Body('auth_type', type=str)
    # Security authentication parameter.
    auth_opt = resource.Body('auth_opt', type=dict)
    # Indicates whether CORS is supported.
    # TRUE: supported
    # FALSE: not supported
    cors = resource.Body('cors', type=bool)
    # API matching mode:
    # SWA: Prefix match
    # NORMAL: Exact match Default value: NORMAL
    match_mode = resource.Body('match_mode', type=str)
    # Backend type. Options:
    # HTTP: web backend
    # FUNCTION: FunctionGraph backend. When backend_type is set to FUNCTION,
    # the func_info field is mandatory.
    # MOCK: mock backend. When backend_type is set to MOCK,
    # the mock_info field is mandatory.
    # GRPC: gRPC backend.
    backend_type = resource.Body('backend_type', type=str)
    # API description. It cannot exceed 255 characters.
    remark = resource.Body('remark', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API request body
    body_remark = resource.Body('body_remark', type=str)
    # Example response for a successful request.
    # Ensure that the response does not exceed 20,480 characters.
    result_normal_sample = resource.Body('result_normal_sample', type=str)
    # Example response for a failed request.
    # Ensure that the response does not exceed 20,480 characters.
    result_failure_sample = resource.Body('result_failure_sample', type=str)
    # Custom authorizer ID.
    # This is unavailable if the request protocol is set to GRPC.
    authorizer_id = resource.Body('authorizer_id', type=str)
    # Tags.
    # Use letters, digits, and special characters (-*#%.:_)
    # and start with a letter.
    tags = resource.Body('tags', type=list)
    # App status.
    status = resource.Body('status', type=int)
    # Indicates whether to enable orchestration.
    arrange_necessary = resource.Body('arrange_necessary', type=int)
    # Time when the API is registered.
    register_time = resource.Body('register_time', type=str)
    # Time when the API was last modified.
    update_time = resource.Body('update_time', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Version of the API group to which the API belongs.
    group_version = resource.Body('group_version', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # Time when the API version is published.
    publish_time = resource.Body('publish_time', type=str)
    # Web backend details.
    backend_api = resource.Body('backend_api', type=BackendApi)
    # API group information.
    api_group_info = resource.Body('api_group_info', type=ApiGroupCommonInfo)
    # FunctionGraph backend details.
    func_info = resource.Body('func_info', type=ApiFunc)
    # Mock backend details.
    mock_info = resource.Body('mock_info', type=ApiMock)
    # Request parameters.
    req_params = resource.Body('req_params', type=list, list_type=ReqParam)
    # Backend parameters.
    backend_params = resource.Body(
        'backend_params', type=list, list_type=BackendParam
    )
    # FunctionGraph backend policies.
    policy_functions = resource.Body(
        'policy_functions', type=list, list_type=ApiPolicyFunction
    )
    # Mock backend policies.
    policy_mocks = resource.Body(
        'policy_mocks', type=list, list_type=ApiPolicyMock
    )
    # Web backend policies.
    policy_https = resource.Body(
        'policy_https', type=list, list_type=ApiPolicyHttp
    )
    # Subdomain name that API Gateway automatically allocates to the API group.
    sl_domain = resource.Body('sl_domain', type=str)
    # Subdomain names that APIG automatically allocates to the API group.
    sl_domains = resource.Body('sl_domains', type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestImage(base.BaseFunctionalTest):

    def setUp(self):
        super(TestImage, self).setUp()
        self.ims = self.conn.imsv2

    def test_create_image(self):
        attrs = {
            "name": "CentOS-7-x86_64-GenericCloud.qcow2",
            "description": "Create an image from a file in an OBS bucket",
            "image_url":
                "ims-test-images:rancheros-openstack.img",
            "os_version": "CentOS 7.0 64bit",
            "min_disk": 40,
            "image_tags": [{"key": "key2", "value": "value2"},
                           {"key": "key1", "value": "value1"}]
        }
        result = self.ims.create_image(**attrs)
        self.assertNotEqual(result, None)

    def test_update_and_query_image(self):
        attrs = {
            "name": "CentOS-7-x86_64-GenericCloud.qcow2"
        }
        result_query = self.ims.images(**attrs)
        self.assertNotEqual(result_query, None)

        image_id = ""
        for image in result_query:
            image_id = image['id']
        command_list = [
            {
                "op": "replace",
                "path": "/name",
                "value": "NewImageTestName"
            }
        ]
        result_update = self.ims.update_image(
            image_id=image_id,
            command_list=command_list
        )
        self.assertNotEqual(result_update, None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.dis.v2 import stream
from otcextensions.sdk.dis.v2 import stream as sdk_stream
from otcextensions.tests.unit.osclient.dis.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListStreams(fakes.TestDis):

    objects = fakes.FakeStream.create_multiple(3)

    column_list_headers = (
        'Name',
        'Stream Type',
        'Data Type',
        'Partition Count',
        'AutoScale Enabled',
        'Status'
    )

    columns = (
        'name',
        'stream_type',
        'data_type',
        'partition_count',
        'is_auto_scale_enabled',
        'status'
    )

    data = []

    for s in objects:
        data.append((
            s.name,
            s.stream_type,
            s.data_type,
            s.partition_count,
            s.is_auto_scale_enabled,
            s.status
        ))

    def setUp(self):
        super(TestListStreams, self).setUp()

        self.cmd = stream.ListStreams(self.app, None)

        self.client.streams = mock.Mock()
        self.client.api_mock = self.client.streams

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--start-stream-name', '2',
        ]

        verifylist = [
            ('limit', 1),
            ('start_stream_name', '2'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            start_stream_name='2',
        )


class TestCreateStream(fakes.TestDis):

    columns = (
        'auto_scale_max_partition_count',
        'auto_scale_min_partition_count',
        'compression_format',
        'data_duration',
        'data_type',
        'is_auto_scale_enabled',
        'name',
        'partition_count',
        'stream_type',
        'tags'
    )

    def setUp(self):
        super(TestCreateStream, self).setUp()

        self.cmd = stream.CreateStream(self.app, None)

    def test_create(self):
        arglist = [
            'test-stream',
            '--partition-count', '2',
            '--stream-type', 'COMMON',
            '--data-type', 'BLOB',
            '--data-duration', '5',
            '--autoscale-min-count', '6',
            '--autoscale-max-count', '7',
            '--compression-format', 'gzip',
            '--tag', 'key=k1,value=v1',
            '--tag', 'key=k2,value=v2',
            '--autoscale',
        ]
        verifylist = [
            ('streamName', 'test-stream'),
            ('partition_count', 2),
            ('stream_type', 'COMMON'),
            ('data_type', 'BLOB'),
            ('data_duration', 5),
            ('auto_scale_min_partition_count', 6),
            ('auto_scale_max_partition_count', 7),
            ('compression_format', 'gzip'),
            ('tags', [{'key': 'k1', 'value': 'v1'},
                      {'key': 'k2', 'value': 'v2'}]),
            ('autoscale', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        attrs = {}
        attrs.update(
            name='test-stream',
            partition_count=2,
            stream_type='COMMON',
            data_type='BLOB',
            data_duration=5,
            auto_scale_min_partition_count=6,
            auto_scale_max_partition_count=7,
            compression_format='gzip',
            tags=[{'key': 'k1', 'value': 'v1'}, {'key': 'k2', 'value': 'v2'}],
            auto_scale_enabled=True
        )

        self.client.create_stream = mock.Mock(
            return_value=sdk_stream.Stream(**attrs))

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_stream.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestUpdateStreamPartition(fakes.TestDis):

    stream_name = 'test-dis-stream'

    _data = sdk_stream.Stream(
        stream_name=stream_name,
        current_partition_count=4,
        target_partition_count=2
    )

    display_columns = (
        'stream_name',
        'current_partition_count',
        'target_partition_count',
    )

    columns = (
        'name',
        'current_partition_count',
        'target_partition_count',
    )

    data = fakes.gen_data(_data, columns, stream._formatters)

    def setUp(self):
        super(TestUpdateStreamPartition, self).setUp()

        self.cmd = stream.UpdateStreamPartition(self.app, None)

        self.client.update_stream_partition = \
            mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            self._data.name,
            '--partition-count', '2',
        ]
        verifylist = [
            ('streamName', self._data.name),
            ('partition_count', 2),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_stream_partition.assert_called_with(
            'test-dis-stream', 2
        )
        self.assertEqual(self.display_columns, columns)


class TestShowStream(fakes.TestDis):

    _data = fakes.FakeStream.create_one()
    columns = (
        'auto_scale_max_partition_count',
        'auto_scale_min_partition_count',
        'compression_format',
        'created_at',
        'data_type',
        'has_more_partitions',
        'id',
        'is_auto_scale_enabled',
        'name',
        'partitions',
        'readable_partition_count',
        'retention_period',
        'status',
        'stream_id',
        'stream_type',
        'updated_at',
        'writable_partition_count'
    )

    data = fakes.gen_data(_data, columns, stream._formatters)

    def setUp(self):
        super(TestShowStream, self).setUp()

        self.cmd = stream.ShowStream(self.app, None)

        self.client.get_stream = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.name,
        ]

        verifylist = [
            ('streamName', self._data.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_stream.assert_called_with(self._data.name)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_dis_stream',
        ]

        verifylist = [
            ('streamName', 'unexist_dis_stream'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_stream = (
            mock.Mock(side_effect=get_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.get_stream.assert_called_with('unexist_dis_stream')


class TestDeleteStream(fakes.TestDis):

    _data = fakes.FakeStream.create_multiple(2)

    def setUp(self):
        super(TestDeleteStream, self).setUp()

        self.client.delete_stream = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = stream.DeleteStream(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('streamName', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_stream.assert_called_with(self._data[0].name)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for dis_stream in self._data:
            arglist.append(dis_stream.name)

        verifylist = [
            ('streamName', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for dis_stream in self._data:
            calls.append(call(dis_stream.name))
        self.client.delete_stream.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_dis_stream',
        ]
        verifylist = [
            ('streamName', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # delete_mock_result = [None, exceptions.CommandError]

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 DIS Stream(s) failed to delete.', str(e))

        calls = [call(self._data[0].name), call('unexist_dis_stream')]
        self.client.delete_stream.assert_has_calls(calls)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
CBR restore data
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


backup = 'name_or_id_backup'
disk_backup = 'name_or_id_disk_backup'
ecs = 'server_name_or_id'
volume = 'volume_id'

attrs = {
    'mappings': [{
        'backup_id': disk_backup,
        'volume_id': volume
    }],
    'power_on': True,
    'server_id': ecs
}
result = conn.cbr.restore_data(backup=backup, **attrs)
print(result)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''LoadBalancer v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    'pool_ids': sdk_utils.ListOfIdsColumnBR,
    'listener_ids': sdk_utils.ListOfIdsColumnBR,
}

PROVISIONING_STATUS = ['ACTIVE', 'DELETED', 'ERROR', 'PENDING_CREATE',
                       'PENDING_UPDATE', 'PENDING_DELETE']

OPERATING_STATUS = ['ONLINE', 'DRAINING', 'OFFLINE', 'DEGRADED', 'ERROR',
                    'NO_MONITOR']


def _get_columns(item):
    column_map = {
        'is_admin_state_up': 'admin_state_up',
        'listeners': 'listener_ids',
        'pools': 'pool_ids',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListLoadBalancer(command.Lister):
    _description = _('List load balancers')

    columns = (
        'id', 'name', 'project_id', 'vip_address', 'provisioning_status',
        'provider',
    )

    def get_parser(self, prog_name):
        parser = super(ListLoadBalancer, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        data = client.load_balancers()

        return (
            self.columns,
            (utils.get_item_properties(
                s, self.columns, formatters=_formatters
            ) for s in data))


class ShowLoadBalancer(command.ShowOne):
    _description = _('Show the details for a single load balancer')

    def get_parser(self, prog_name):
        parser = super(ShowLoadBalancer, self).get_parser(prog_name)

        parser.add_argument(
            'load_balancer',
            metavar='<load_balancer>',
            help=_("The Name or ID of the Load balancer to show.")
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        obj = client.find_load_balancer(
            name_or_id=parsed_args.load_balancer,
            ignore_missing=False
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateLoadBalancer(command.ShowOne):
    _description = _('Create a load balancer')
    columns = (
        'id', 'name', 'description', 'provisioning_status',
        'operating_status', 'is_admin_state_up',
        'provider', 'pool_ids', 'listener_ids',
        'vip_address', 'vip_subnet_id', 'vip_port_id')

    def get_parser(self, prog_name):
        parser = super(CreateLoadBalancer, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('New load balancer name.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Set load balancer description.')
        )
        parser.add_argument(
            '--vip_address',
            metavar='<vip_address>',
            help=_('Set the VIP IP Address.')
        )
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            '--vip_network_id',
            metavar='<vip_network_id>',
            help=_('Set network for the load balancer (name or ID).')
        )
        group.add_argument(
            '--vip_port_id',
            metavar='<vip_port_id>',
            help=_('Set Port for the load balancer (name or ID).')
        )
        group.add_argument(
            '--vip_subnet_id',
            metavar='<vip_subnet_id>',
            help=_("The ID of the network for the Virtual IP (VIP). "
                   "One of vip_network_id, vip_port_id, or vip_subnet_id "
                   "must be specified.")
        )
        parser.add_argument(
            '--vip_qos_policy_id',
            metavar='<vip_qos_policy_id>',
            help=_('Set QoS policy ID for VIP port. Unset with ''None''.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable load balancer (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable load balancer.')
        )
        # TODO(agoncharov) a full blown listeners structure is supported by API
        # parser.add_argument(
        #     '--listeners',
        #     metavar='<listeners>',
        #     help=_("The associated listener IDs, if any.")
        # )

        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.disable:
            args['is_admin_state_up'] = False
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.vip_address:
            args['vip_address'] = parsed_args.vip_address
        if parsed_args.vip_network_id:
            args['vip_network_id'] = parsed_args.vip_network_id
        if parsed_args.vip_port_id:
            args['vip_port_id'] = parsed_args.vip_port_id
        if parsed_args.vip_subnet_id:
            args['vip_subnet_id'] = parsed_args.vip_subnet_id
        if parsed_args.vip_qos_policy_id:
            args['vip_qos_policy_id'] = parsed_args.vip_qos_policy_id

        client = self.app.client_manager.network

        obj = client.create_load_balancer(**args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class SetLoadBalancer(command.ShowOne):
    _description = _('Update a load balancer')
    columns = (
        'ID', 'Name', 'description',
        'provisioning_status', 'operating_status', 'is_admin_state_up',
        'provider', 'pool_ids', 'listener_ids',
        'vip_address', 'vip_subnet_id', 'vip_port_id')

    def get_parser(self, prog_name):
        parser = super(SetLoadBalancer, self).get_parser(prog_name)

        parser.add_argument(
            'load_balancer',
            metavar='<load_balancer>',
            help=_("The ID for the load balancer.")
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=None,
            help="Enable load balancer."
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help="Disable load balancer."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("A human-readable description for the resource.")
        )

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Human-readable name of the resource.")
        )
        parser.add_argument(
            '--vip_qos_policy_id',
            metavar='<vip_qos_policy_id>',
            help=_("The ID of the QoS Policy which will apply to "
                   "the Virtual IP (VIP).")
        )

        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.disable:
            args['is_admin_state_up'] = False
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.vip_qos_policy_id:
            args['vip_qos_policy_id'] = parsed_args.vip_qos_policy_id

        client = self.app.client_manager.network

        obj = client.update_load_balancer(
            load_balancer=parsed_args.load_balancer,
            **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteLoadBalancer(command.Command):
    _description = _('Delete a load balancer')

    def get_parser(self, prog_name):
        parser = super(DeleteLoadBalancer, self).get_parser(prog_name)

        parser.add_argument(
            'load_balancer',
            metavar='<load_balancer>',
            nargs='+',
            help=_("The ID for the load balancer.")
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        for lb in parsed_args.load_balancer:
            obj = client.find_load_balancer(name_or_id=lb)
            client.delete_load_balancer(
                load_balancer=obj.id,
                ignore_missing=False
            )

        return



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''OBS Object v1 action implementations'''
import logging
import os

from otcextensions.i18n import _

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

_file_hash_cache = dict()


def _get_columns(item):
    column_map = {
        'Content Type': 'content_type',
        'Last Modified': 'last_modified_at',
        'Hash': 'etag',
        'Bytes': 'content_length',
        'Accept Ranges': 'accept_ranges',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateObject(command.ShowOne):
    _description = _('Upload object to container')

    def get_parser(self, prog_name):
        parser = super(CreateObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container for new object'),
        )
        parser.add_argument(
            'objects',
            metavar='<filename>',
            nargs='+',
            help=_('Local filename(s) to upload'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Upload a file and rename it. '
                   'Can only be used when uploading a single object')
        )
        parser.add_argument(
            '--name-prefix',
            metavar='<prefix>',
            help=_('Object name prefix. '
                   'Useful when uploading multiple objects'),
            default=''
        )
        return parser

    def take_action(self, parsed_args):
        if parsed_args.name:
            if len(parsed_args.objects) > 1:
                msg = _('Attempting to upload multiple objects and '
                        'using --name is not permitted')
                raise exceptions.CommandError(msg)
        results = []
        for obj in parsed_args.objects:
            if len(obj) > 1024:
                LOG.warning(
                    _('Object name is %s characters long, default limit'
                      ' is 1024'), len(obj))
            prefix = parsed_args.name_prefix + os.path.basename(obj)
            data = self.app.client_manager.obs.create_object(
                container=parsed_args.container,
                name=parsed_args.name or prefix,
                data=open(obj, 'r').read()
            )
            results.append(data)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)
        return (display_columns, data)


class DeleteObject(command.Command):
    _description = _('Delete object from container')

    def get_parser(self, prog_name):
        parser = super(DeleteObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Delete object(s) from <container>'),
        )
        parser.add_argument(
            'objects',
            metavar='<object>',
            nargs='+',
            help=_('Object(s) to delete'),
        )
        return parser

    def take_action(self, parsed_args):

        for obj in parsed_args.objects:
            self.app.client_manager.obs.delete_object(
                container=parsed_args.container,
                obj=obj,
            )


class ListObject(command.Lister):
    _description = _('List objects')

    def get_parser(self, prog_name):
        parser = super(ListObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container to list'),
        )
        parser.add_argument(
            '--prefix',
            metavar='<prefix>',
            help=_('Filter list using <prefix>'),
        )
        parser.add_argument(
            '--delimiter',
            metavar='<delimiter>',
            help=_('Roll up items with <delimiter>'),
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('Anchor for paging'),
        )
        parser.add_argument(
            '--end-marker',
            metavar='<end-marker>',
            help=_('End anchor for paging'),
        )
        parser.add_argument(
            '--limit',
            metavar='<num-objects>',
            type=int,
            help=_('Limit the number of objects returned'),
        )
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_('List additional fields in output'),
        )
        parser.add_argument(
            '--all',
            action='store_true',
            default=False,
            help=_('List all objects in container (default is 10000)'),
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.long:
            columns = (
                'name',
                'content length',
                'etag',
                # 'Content Type',
                'last modified',
            )
        else:
            columns = ('name',)

        kwargs = {}
        if parsed_args.prefix:
            kwargs['prefix'] = parsed_args.prefix
        if parsed_args.delimiter:
            kwargs['delimiter'] = parsed_args.delimiter
        if parsed_args.marker:
            kwargs['marker'] = parsed_args.marker
        if parsed_args.end_marker:
            kwargs['end_marker'] = parsed_args.end_marker
        if parsed_args.limit:
            kwargs['limit'] = parsed_args.limit
        if parsed_args.all:
            kwargs['full_listing'] = True

        data = self.app.client_manager.obs.objects(
            container=parsed_args.container,
            **kwargs
        )

        return (columns,
                (utils.get_item_properties(
                    s, columns,
                    formatters={},
                ) for s in data))


class SaveObject(command.Command):
    _description = _('Save object locally')

    def get_parser(self, prog_name):
        parser = super(SaveObject, self).get_parser(prog_name)
        parser.add_argument(
            '--file',
            metavar='<filename>',
            default='-',
            help=_('Destination filename (defaults to object name); using "-"'
                   ' as the filename will print the file to stdout'),
        )
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Download <object> from <container>'),
        )
        parser.add_argument(
            'object',
            metavar='<object>',
            help=_('Object to save'),
        )
        return parser

    def take_action(self, parsed_args):

        self.app.client_manager.obs.download_object(
            container=parsed_args.container,
            obj=parsed_args.object,
            file=parsed_args.file
        )


class SetObject(command.Command):
    _description = _('Set object properties')

    def get_parser(self, prog_name):
        parser = super(SetObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Modify <object> from <container>'),
        )
        parser.add_argument(
            'object',
            metavar='<object>',
            help=_('Object to modify'),
        )
        parser.add_argument(
            '--property',
            metavar='<key=value>',
            required=True,
            action=parseractions.KeyValueAction,
            help=_('Set a property on this object '
                   '(repeat option to set multiple properties)')
        )
        return parser

    def take_action(self, parsed_args):
        self.app.client_manager.obs.object_set(
            parsed_args.container,
            parsed_args.object,
            properties=parsed_args.property,
        )


class ShowObject(command.ShowOne):
    _description = _('Display object details')

    def get_parser(self, prog_name):
        parser = super(ShowObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Display <object> from <container>'),
        )
        parser.add_argument(
            'object',
            metavar='<object>',
            help=_('Object to display'),
        )
        return parser

    def take_action(self, parsed_args):

        data = self.app.client_manager.obs.get_object(
            container=parsed_args.container,
            obj=parsed_args.object,
        )

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return (display_columns, data)


class UnsetObject(command.Command):
    _description = _('Unset object properties')

    def get_parser(self, prog_name):
        parser = super(UnsetObject, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Modify <object> from <container>'),
        )
        parser.add_argument(
            'object',
            metavar='<object>',
            help=_('Object to modify'),
        )
        parser.add_argument(
            '--property',
            metavar='<key>',
            required=True,
            action='append',
            default=[],
            help=_('Property to remove from object '
                   '(repeat option to remove multiple properties)'),
        )
        return parser

    def take_action(self, parsed_args):
        self.app.client_manager.obs.object_unset(
            parsed_args.container,
            parsed_args.object,
            properties=parsed_args.property,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.elb.v2 import load_balancer_tag

EXAMPLE = {
    "key": 'fake_key',
    "value": "fake_name",
}


class TestElbLoadBalancerTag(base.TestCase):

    def setUp(self):
        super(TestElbLoadBalancerTag, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = load_balancer_tag.Tag()

        self.assertEqual(
            '/loadbalancers/%(loadbalancer_id)s/tags',
            sot.base_path)
        self.assertEqual('tags', sot.resources_key)
        self.assertEqual('tag', sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make(self):
        sot = load_balancer_tag.Tag(**EXAMPLE)
        self.assertEqual(EXAMPLE['key'], sot.key)
        self.assertEqual(EXAMPLE['value'], sot.value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionAsyncNotification(TestFg):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionAsyncNotification, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        enable = self.conn.functiongraph.configure_async_notification(
            self.function,
            max_async_event_age_in_seconds=1000,
            # enable_async_status_log=True
        )
        self.assertIsNotNone(enable)

        self.inv = self.conn.functiongraph.executing_function_asynchronously(
            self.function.func_urn, attrs={'a': 'b'}
        )
        self.assertIsNotNone(self.inv.request_id)

        self.addCleanup(
            self.client.delete_function,
            self.function
        )
        self.addCleanup(
            self.client.delete_async_notification,
            self.function
        )

    def test_async_notifications(self):
        notifications = list(self.client.async_notifications(
            self.function.func_urn))
        self.assertIn(self.function.func_urn, notifications[0].func_urn)

    def test_all_versions_async_notifications(self):
        notifications = list(self.client.all_versions_async_notifications(
            self.function.func_urn))
        self.assertIn(self.function.func_urn, notifications[0].func_urn)

    def test_async_invocation_requests(self):
        requests = list(self.client.async_invocation_requests(
            self.function, **{'request_id': self.inv.request_id}))
        self.assertIsNotNone(requests)

    # def test_stop_async_invocation_request(self):
    #     stop = self.client.stop_async_invocation_request(
    #         self.function, request_id=self.inv.request_id)
    #     self.assertIsNone(stop)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.cce.v3 import _proxy
from otcextensions.sdk.cce.v3 import cluster as _cluster
from otcextensions.sdk.cce.v3 import cluster_node as _cluster_node
from otcextensions.sdk.cce.v3 import node_pool as _node_pool
from otcextensions.sdk.cce.v3 import cluster_cert as _cluster_cert
from otcextensions.sdk.cce.v3 import job as _job


class TestCCEProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestCCEProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestCCECluster(TestCCEProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.clusters, _cluster.Cluster,
            # method_kwargs={},
            expected_kwargs={
                'paginated': False
            }
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_cluster,
            _cluster.Cluster,
            expected_kwargs={}
        )

    def test_create(self):
        self.verify_create(
            self.proxy.create_cluster, _cluster.Cluster,
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'instance': 'test',
                'name': 'some_name'
            }
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_cluster,
            _cluster.Cluster, True,
            expected_kwargs={}
        )

    def test_get_certs(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_cluster_certificates,
            method_args=[cluster],
            expected_args=[_cluster_cert.ClusterCertificate],
            expected_kwargs={
                'cluster_id': cluster.id,
                'requires_id': False
            }
        )


class TestCCEClusterNode(TestCCEProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.cluster_nodes, _cluster_node.ClusterNode,
            method_args=[_cluster.Cluster(id='cluster_id')],
            method_kwargs={},
            expected_kwargs={
                'paginated': False,
                'cluster_id': 'cluster_id'
            },
            expected_args=[]
        )

    def test_get(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_cluster_node,
            method_args=[cluster, 'node'],
            expected_args=[_cluster_node.ClusterNode, 'node'],
            expected_kwargs={
                'cluster_id': cluster.id
            }
        )

    def test_find(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_cluster_node,
            method_args=[cluster, 'node'],
            expected_args=[_cluster_node.ClusterNode, 'node'],
            expected_kwargs={
                'cluster_id': cluster.id
            }
        )

    def test_add_node(self):
        attrs = {'a': 'b'}
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._create',
            self.proxy.create_cluster_node,
            method_args=[cluster],
            method_kwargs=attrs,
            expected_args=[_cluster_node.ClusterNode],
            expected_kwargs=dict(
                cluster_id=cluster.id,
                **attrs
            )
        )

    def test_delete_node(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_cluster_node,
            method_args=[cluster, 'n1'],
            expected_args=[_cluster_node.ClusterNode, 'n1'],
            expected_kwargs={
                'cluster_id': cluster.id,
                'ignore_missing': True
            }
        )

    def test_server_wait_for(self):
        value = _cluster.Cluster(id='cluster_id')
        self.verify_wait_for_status(
            self.proxy.wait_for_cluster,
            method_args=[value],
            expected_args=[self.proxy, value, 'Available', ['ERROR'], 2, 960],
            expected_kwargs={'attribute': 'status.status'})


class TestCCENodePool(TestCCEProxy):

    def test_list_node_pool(self):
        self.verify_list(
            self.proxy.node_pools, _node_pool.NodePool,
            method_args=[_cluster.Cluster(id='cluster_id')],
            method_kwargs={},
            expected_kwargs={
                'paginated': False,
                'cluster_id': 'cluster_id'
            },
            expected_args=[]
        )

    def test_get_node_pool(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_node_pool,
            method_args=[cluster, 'node_pool'],
            expected_args=[_node_pool.NodePool, 'node_pool'],
            expected_kwargs={
                'cluster_id': cluster.id
            }
        )

    def test_find_node_pool(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_node_pool,
            method_args=[cluster, 'node_pool'],
            expected_args=[_node_pool.NodePool, 'node_pool'],
            expected_kwargs={
                'cluster_id': cluster.id
            }
        )

    def test_add_node_pool(self):
        attrs = {'a': 'b'}
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._create',
            self.proxy.create_node_pool,
            method_args=[cluster],
            method_kwargs=attrs,
            expected_args=[_node_pool.NodePool],
            expected_kwargs=dict(
                cluster_id=cluster.id,
                **attrs
            )
        )

    def test_delete_node_pool(self):
        cluster = _cluster.Cluster(id='cluster_id')
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_node_pool,
            method_args=[cluster, 'n1'],
            expected_args=[_node_pool.NodePool, 'n1'],
            expected_kwargs={
                'cluster_id': cluster.id,
                'ignore_missing': True
            }
        )


class TestCCEJob(TestCCEProxy):
    def test_get(self):
        self.verify_get(
            self.proxy.get_job,
            _job.Job,
            expected_kwargs={}
        )

    def test_wait_for_job(self):
        self.proxy.get_job = mock.Mock(return_value='some_fake')
        self.verify_wait_for_status(
            self.proxy.wait_for_job,
            method_args=['fake_job_id'],
            expected_args=[
                self.proxy, 'some_fake', 'success', ['FAILED'], 5, 3600
            ],
            expected_kwargs={'attribute': 'status.status'}
        )
        self.proxy.get_job.assert_called_with('fake_job_id')


class TestExtractName(TestCCEProxy):

    def test_extract_name(self):

        self.assertEqual(
            ['discovery'],
            self.proxy._extract_name('/api/v3/projects/123', project_id='123')
        )
        self.assertEqual(
            ['clusters'],
            self.proxy._extract_name(
                '/api/v3/projects/123/clusters', project_id='123')
        )
        self.assertEqual(
            ['cluster'],
            self.proxy._extract_name(
                '/api/v3/projects/123/clusters/c1', project_id='123')
        )
        self.assertEqual(
            ['cluster', 'clustercert'],
            self.proxy._extract_name(
                '/api/v3/projects/123/clusters/c1/clustercert',
                project_id='123')
        )
        self.assertEqual(
            ['cluster', 'nodes'],
            self.proxy._extract_name(
                '/api/v3/projects/123/clusters/c1/nodes', project_id='123')
        )
        self.assertEqual(
            ['cluster', 'node'],
            self.proxy._extract_name(
                '/api/v3/projects/123/clusters/c1/nodes/n1', project_id='123')
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.deh.v1 import server


FAKE_ID = '68d5745e-6af2-40e4-945d-fe449be00148'
EXAMPLE = {
    'addresses': {
        '68269e6e-4a27-441b-8029-35373ad50bd9': [
            {
                'addr': '192.168.0.3',
                'version': 4
            }
        ]
    },
    'created': '2012-09-07T16:56:37Z',
    'flavor': {
        'id': '1'
    },
    'id': FAKE_ID,
    'metadata': {
        'os_type': 'Linux'
    },
    'name': 'new-server-test',
    'status': 'ACTIVE',
    'tenant_id': 'openstack',
    'updated': '2012-09-07T16:56:37Z',
    'user_id': 'fake'
}


class TestHost(base.TestCase):

    def test_basic(self):
        sot = server.Server()

        self.assertEqual('/dedicated-hosts/%(dedicated_host_id)s/servers',
                         sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):

        sot = server.Server(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], FAKE_ID)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
        self.assertEqual(EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import typing as ty

from openstack import exceptions
from openstack import resource
from openstack import utils
from otcextensions.common import format as otc_format


class Snapshot(resource.Resource):
    base_path = '/clusters/%(uri_cluster_id)s/index_snapshot'

    resource_key = 'backup'
    resources_key = 'backups'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_fetch = True

    #: ID of the cluster where index data is to be backed up.
    uri_cluster_id = resource.URI('uri_cluster_id')

    # Properties
    #: Snapshot retention period.
    backup_keep_days = resource.Body('backupKeepDay', type=int)
    #: Snapshot creation mode.
    backup_method = resource.Body('backupMethod')
    #: Time when a snapshot is executed every day.
    backup_period = resource.Body('backupPeriod')
    #: Time when the snapshot starts to be executed.
    backup_start_time = resource.Body('backupExpectedStartTime')
    #: Snapshot Type Automatic/Manual
    backup_type = resource.Body('backupType')
    #: Bucket for storing snapshot data.
    bucket_name = resource.Body('bucketName')
    #: Cluster name.
    cluster_name = resource.Body('clusterName')
    #: Cluster Id.
    cluster_id = resource.Body('clusterId')
    #: Time when a snapshot is created.
    created_at = resource.Body('created')
    #: Type of the data search engine.
    datastore = resource.Body('datastore', type=dict)
    #: Description of a snapshot.
    description = resource.Body('description')
    #: Timestamp when the snapshot execution ends.
    end_time = resource.Body('endTime')
    #: Number of shards that fail to be backed up.
    failed_shards = resource.Body('failedShards', type=int)
    #: Indices that need to be backed up.
    indices = resource.Body('indices')
    #: Rule for defining the indices to be restored.
    rename_pattern = resource.Body('renamePattern')
    #: Rule for renaming an index.
    rename_replacement = resource.Body('renameReplacement')
    #: Snapshot restoration status.
    restore_status = resource.Body('restoreStatus')
    #: Timestamp when the snapshot starts to be executed.
    start_time = resource.Body('startTime')
    #: Snapshot status.
    status = resource.Body('status')
    #: ID of the cluster, to which the snapshot is to be restored.
    target_cluster = resource.Body('targetCluster')
    #: Total number of shards of the indices to be backed up.
    total_shards = resource.Body('totalShards', type=int)
    #: Time when a snapshot status is updated.
    updated_at = resource.Body('updated')
    #: Version of the snapshot.
    version = resource.Body('version')

    @classmethod
    def find(
        cls,
        session,
        name_or_id: str,
        ignore_missing: bool = True,
        list_base_path: ty.Optional[str] = None,
        *,
        microversion: ty.Optional[str] = None,
        all_projects: ty.Optional[bool] = None,
        **params,
    ):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
            the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the resource does not exist.  When set to ``True``, None will be
            returned when attempting to find a nonexistent resource.
        :param str list_base_path: base_path to be used when need listing
            resources.
        :param str microversion: API version to override the negotiated one.
        :param dict params: Any additional parameters to be passed into
            underlying methods, such as to
            :meth:`~openstack.resource.Resource.existing` in order to pass on
            URI parameters.

        :return: The :class:`Resource` object matching the given name or id
            or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
            than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
            is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)

        if list_base_path:
            params['base_path'] = list_base_path

        # all_projects is a special case that is used by multiple services. We
        # handle it here since it doesn't make sense to pass it to the .fetch
        # call above
        if all_projects is not None:
            params['all_projects'] = all_projects

        if (
            'name' in cls._query_mapping._mapping.keys()
            and 'name' not in params
        ):
            params['name'] = name_or_id

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None

        raise exceptions.ResourceNotFound(
            f'No {cls.__name__} found for {name_or_id}'
        )

    def create(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # backup create doesn't accept resource_key in its request.
        return super(Snapshot, self).create(
            session, prepend_key=False, base_path=base_path
        )

    def restore(self, session, cluster_id, **body):
        """Restoring a Snapshot."""
        uri = utils.urljoin(
            'clusters', cluster_id, 'index_snapshot', self.id, 'restore'
        )
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)


class SnapshotPolicy(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/index_snapshot/policy'

    allow_create = True
    allow_fetch = True

    #: ID of the cluster where automatic snapshot creation is enabled.
    cluster_id = resource.URI('cluster_id')

    #: Agency used to access OBS buckets.
    agency = resource.Body('agency')
    #: Storage path of the snapshot in the OBS bucket.
    backup_path = resource.Body('basePath')
    #: Time when a snapshot is created every day.
    backup_period = resource.Body('period')
    #: Retention days for a snapshot.
    backup_keep_days = resource.Body('keepday', type=int)
    #: OBS bucket for storing snapshots.
    bucket_name = resource.Body('bucket')
    #: Snapshot encryption ID.
    cmk_id = resource.Body('snapshotCmkId')
    #: Whether to delete all automatically created snapshots when the
    #:  automatic snapshot creation policy is disabled.
    delete_auto = resource.Body('deleteAuto', type=otc_format.BoolStr_1)
    #: Frequency of automatically creating snapshots.
    frequency = resource.Body('frequency')
    #: Snapshot name prefix.
    prefix = resource.Body('prefix')
    #: Name of the index to be backed up.
    indices = resource.Body('indices')
    #: Whether to enable the automatic snapshot creation policy.
    is_enabled = resource.Body('enable', type=otc_format.BoolStr_1)


class SnapshotConfiguration(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/index_snapshot/%(setting)s'

    allow_create = True

    #: ID of the cluster where automatic snapshot creation is enabled.
    cluster_id = resource.URI('cluster_id')
    #: Setting -> auto_setting or custom setting
    setting = resource.URI('setting')

    #: Storage path of the snapshot in the OBS bucket.
    backup_path = resource.Body('basePath')
    #: OBS bucket used for index data backup.
    bucket_name = resource.Body('bucket')
    #: IAM agency used to access OBS.
    agency = resource.Body('agency')
    #: Key ID used for snapshot encryption.
    cmk_id = resource.Body('snapshotCmkId')

    def disable(self, session):
        """Disable Snapshot Function."""
        uri = utils.urljoin('clusters', self.cluster_id, 'index_snapshots')
        response = session.delete(uri)
        exceptions.raise_from_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Route(resource.Resource):
    resources_key = 'routes'
    resource_key = 'route'
    base_path = '/v2.0/vpc/routes'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'marker', 'limit', 'type', 'router_id',
        'destination', 'project_id', project_id='tenant_id',
        router_id='vpc_id'
    )

    #: Specifies the VPC peering connection ID.
    #: *Type: uuid*
    id = resource.Body('id')
    #: Specifies the next hop.
    #:  If the route type is peering, enter the VPC peering connection ID.
    nexthop = resource.Body('nexthop')
    #: Specifies the destination address in the CIDR notation format,
    #:  for example, 192.168.200.0/24.
    destination = resource.Body('destination')
    #: Specifies the route type. Currently, the value can only be peering.
    type = resource.Body('type')
    #: Specifies the Router of the route.
    #:  Set this parameter to the existing Router ID.
    router_id = resource.Body('vpc_id')
    #: Specifies the project ID.
    project_id = resource.Body('tenant_id')



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a dataset from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
attrs = {
    "workspace_id": "0",
    "dataset_name": "dataset-test",
    "dataset_type": 0,
    "data_sources": [
        {
            "data_type": 0,
            "data_path": "/fkukucsk/outp/",
        }
    ],
    "description": "",
    "work_path": "/fkukucsk/outpd/",
    "work_path_type": 0,
    "ai_project": "",
}

dataset = conn.modelartsv2.create_dataset(**attrs)
print(dataset)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import service_log
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestServiceLogs(fakes.TestModelartsv1):
    _service = fakes.FakeService.create_one()
    objects = fakes.FakeServiceLog.create_multiple(3)

    column_list_headers = (
        "Update Time",
        "Result",
        "Config",
        "Result Detail",
    )

    data = []

    for s in objects:
        data.append(
            (
                cli_utils.UnixTimestampFormatter(s.update_time),
                s.result,
                cli_utils.YamlFormat(s.config),
                s.result_detail,
            )
        )

    def setUp(self):
        super(TestServiceLogs, self).setUp()

        self.cmd = service_log.ServiceLogs(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._service)
        self.client.service_logs = mock.Mock()
        self.client.api_mock = self.client.service_logs

    def test_list(self):
        arglist = [self._service.name]

        verifylist = [
            ("service", self._service.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_service.assert_called_with(
            self._service.name, ignore_missing=False
        )
        self.client.api_mock.assert_called_with(self._service.id)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            self._service.name,
            "--update-time",
            "123",
        ]

        verifylist = [
            ("service", self._service.name),
            ("update_time", 123),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._service.id,
            update_time=123,
        )
